"""medication knowledge serializers."""

from drf_writable_nested import WritableNestedModelSerializer

from dfhir.base.serializers import (
    AnnotationSerializer,
    AttachmentSerializer,
    BaseReferenceModelSerializer,
    BaseWritableNestedModelSerializer,
    CodeableConceptSerializer,
    ContactDetailSerializer,
    DurationSerializer,
    IdentifierSerializer,
    MoneySerializer,
    OrganizationReferenceSerializer,
    PeriodSerializer,
    RatioSerializer,
    SimpleQuantitySerializer,
)
from dfhir.documentreferences.serializers import DocumentReferenceReferenceSerializer
from dfhir.medicationknowledges.models import (
    MedicationKnowledge,
    MedicationKnowledgeCost,
    MedicationKnowledgeDefinitional,
    MedicationKnowledgeDefinitionalDrugCharacteristic,
    MedicationKnowledgeDefinitionalIngredient,
    MedicationKnowledgeIndicationGuideline,
    MedicationKnowledgeIndicationGuidelineDosingGuideline,
    MedicationKnowledgeIndicationGuidelineDosingGuidelineDosage,
    MedicationKnowledgeMedicineClassification,
    MedicationKnowledgeMonitoringProgram,
    MedicationKnowledgeMonograph,
    MedicationKnowledgePackaging,
    MedicationKnowledgePatientCharacteristics,
    MedicationKnowledgeReference,
    MedicationKnowledgeRegulatory,
    MedicationKnowledgeRegulatoryMaxDispense,
    MedicationKnowledgeRegulatorySubstitution,
    MedicationKnowledgeRelatedMedicationKnowledge,
    MedicationKnowledgeStorageGuideline,
    MedicationKnowledgeStorageGuidelineEnvironmentalSetting,
)
from dfhir.medications.serializers import MedicationReferenceSerializer
from dfhir.substances.serializers import SubstanceCodeableReferenceSerializer


class MedicationKnowledgeReferenceSerializer(BaseReferenceModelSerializer):
    """Medication knowledge reference serializer."""

    identifier = IdentifierSerializer(required=False)

    class Meta:
        """Meta class."""

        model = MedicationKnowledgeReference
        exclude = ["created_at", "updated_at"]


class MedicationKnowledgeRelatedMedicationKnowledgeSerializer(
    WritableNestedModelSerializer
):
    """medication knowledge related medication knowledge serializer."""

    type = CodeableConceptSerializer(many=False, required=False)
    reference = MedicationKnowledgeReferenceSerializer(many=False, required=False)

    class Meta:
        """meta options."""

        model = MedicationKnowledgeRelatedMedicationKnowledge
        exclude = ["created_at", "updated_at"]


class MedicationKnowledgeMonographSerializer(WritableNestedModelSerializer):
    """Medication knowledge monograph serializer."""

    type = CodeableConceptSerializer(many=False, required=False)
    source = DocumentReferenceReferenceSerializer(required=False)

    class Meta:
        """meta options."""

        model = MedicationKnowledgeMonograph
        exclude = ["created_at", "updated_at"]


class MedicationKnowledgeCostSerializer(WritableNestedModelSerializer):
    """Medication knowledge cost serializer."""

    effective_date = PeriodSerializer(required=False, many=True)
    type = CodeableConceptSerializer(many=False, required=False)
    source = DocumentReferenceReferenceSerializer(required=False)
    cost_money = MoneySerializer(required=False)
    cost_codeable_concept = CodeableConceptSerializer(required=False)

    class Meta:
        """meta options."""

        model = MedicationKnowledgeCost
        exclude = ["created_at", "updated_at"]


class MedicationKnowledgeMonitoringProgramSerializer(WritableNestedModelSerializer):
    """Medication knowledge monitoring program serializer."""

    type = CodeableConceptSerializer(many=False, required=False)

    class Meta:
        """meta options."""

        model = MedicationKnowledgeMonitoringProgram
        exclude = ["created_at", "updated_at"]


class MedicationKnowledgePatientCharacteristicsSerializer(
    WritableNestedModelSerializer
):
    """Medication knowledge dosing guideline treatment intent dosage patient characteristics serializer."""

    type = CodeableConceptSerializer(many=False, required=False)
    value_codeable_concept = CodeableConceptSerializer(many=False, required=False)
    value_quantity = MoneySerializer(many=False, required=False)
    value_range = MoneySerializer(many=False, required=False)

    class Meta:
        """meta options."""

        model = MedicationKnowledgePatientCharacteristics
        exclude = ["created_at", "updated_at"]


class MedicationKnowledgeIndicationGuidelineDosingGuidelineDosageSerializer(
    WritableNestedModelSerializer
):
    """Medication knowledge dosing guideline dosage serializer."""

    type = CodeableConceptSerializer(many=False, required=False)
    # TODO: dose = DoseSerializer(many=False, required=False)
    administration_treatment = CodeableConceptSerializer(many=False, required=False)

    class Meta:
        """meta options."""

        model = MedicationKnowledgeIndicationGuidelineDosingGuidelineDosage
        exclude = ["created_at", "updated_at"]


class MedicationKnowledgeIndicationGuidelineDosingGuidelineSerializer(
    WritableNestedModelSerializer
):
    """Medication knowledge indication guideline dosing guideline serializer."""

    treatment_intent = CodeableConceptSerializer(many=False, required=False)
    dosage = MedicationKnowledgeIndicationGuidelineDosingGuidelineDosageSerializer(
        many=True, required=False
    )
    administration_treatment = CodeableConceptSerializer(many=False, required=False)
    patient_characteristics = MedicationKnowledgePatientCharacteristicsSerializer(
        many=True, required=False
    )

    class Meta:
        """meta options."""

        model = MedicationKnowledgeIndicationGuidelineDosingGuideline
        exclude = ["created_at", "updated_at"]


class MedicationKnowledgeIndicationGuidelineSerializer(WritableNestedModelSerializer):
    """Medication knowledge indication guideline serializer."""

    dosing_guideline = MedicationKnowledgeIndicationGuidelineDosingGuidelineSerializer(
        many=True, required=False
    )

    class Meta:
        """meta options."""

        model = MedicationKnowledgeIndicationGuideline
        exclude = ["created_at", "updated_at"]


class MedicationKnowledgeMedicineClassificationSerializer(
    WritableNestedModelSerializer
):
    """Medication knowledge medicine classification serializer."""

    type = CodeableConceptSerializer(many=False, required=False)
    classification = CodeableConceptSerializer(many=True, required=False)

    class Meta:
        """meta options."""

        model = MedicationKnowledgeMedicineClassification
        exclude = ["created_at", "updated_at"]


class MedicationKnowledgePackagingSerializer(WritableNestedModelSerializer):
    """Medication knowledge packaging serializer."""

    cost = MedicationKnowledgeCostSerializer(many=True, required=False)

    class Meta:
        """meta options."""

        model = MedicationKnowledgePackaging
        exclude = ["created_at", "updated_at"]


class MedicationKnowledgeStorageGuidelineEnvironmentalSettingSerializer(
    WritableNestedModelSerializer
):
    """Medication knowledge storage guideline environmental setting serializer."""

    type = CodeableConceptSerializer(many=False, required=False)
    value_quantity = MoneySerializer(many=False, required=False)
    value_range = MoneySerializer(many=False, required=False)
    value_codeable_concept = CodeableConceptSerializer(many=False, required=False)

    class Meta:
        """meta options."""

        model = MedicationKnowledgeStorageGuidelineEnvironmentalSetting
        exclude = ["created_at", "updated_at"]


class MedicationKnowledgeStorageGuidelineSerializer(WritableNestedModelSerializer):
    """Medication knowledge storage guideline serializer."""

    note = AnnotationSerializer(many=True, required=False)
    stability_duration = DurationSerializer(required=False)
    environmental_setting = (
        MedicationKnowledgeStorageGuidelineEnvironmentalSettingSerializer(
            many=True, required=False
        )
    )

    class Meta:
        """meta options."""

        model = MedicationKnowledgeStorageGuideline
        exclude = ["created_at", "updated_at"]


class MedicationKnowledgeRegulatorySubstitutionSerializer(
    WritableNestedModelSerializer
):
    """Medication knowledge regulatory substitution serializer."""

    type = CodeableConceptSerializer(many=False, required=False)

    class Meta:
        """meta options."""

        model = MedicationKnowledgeRegulatorySubstitution
        exclude = ["created_at", "updated_at"]


class MedicationKnowledgeRegulatoryMaxDispenseSerializer(WritableNestedModelSerializer):
    """Medication knowledge regulatory max dispense serializer."""

    quantity = SimpleQuantitySerializer(required=False)
    period = PeriodSerializer(required=False)

    class Meta:
        """meta options."""

        model = MedicationKnowledgeRegulatoryMaxDispense
        exclude = ["created_at", "updated_at"]


class MedicationKnowledgeRegulatorySerializer(WritableNestedModelSerializer):
    """Medication knowledge regulatory serializer."""

    regulatory_authority = OrganizationReferenceSerializer(many=False, required=False)
    substitution = MedicationKnowledgeRegulatorySubstitutionSerializer(
        many=True, required=False
    )
    schedule = CodeableConceptSerializer(many=False, required=False)
    max_dispense = MedicationKnowledgeRegulatoryMaxDispenseSerializer(
        many=True, required=False
    )

    class Meta:
        """meta options."""

        model = MedicationKnowledgeRegulatory
        exclude = ["created_at", "updated_at"]


class MedicationKnowledgeDefinitionalIngredientSerializer(
    WritableNestedModelSerializer
):
    """Medication knowledge definitional ingredient serializer."""

    item = SubstanceCodeableReferenceSerializer(many=False, required=False)
    type = CodeableConceptSerializer(many=False, required=False)
    strength_ratio = RatioSerializer(many=False, required=False)
    strength_quantity = SimpleQuantitySerializer(many=False, required=False)
    strength_codeable_concept = CodeableConceptSerializer(many=False, required=False)

    class Meta:
        """meta options."""

        model = MedicationKnowledgeDefinitionalIngredient
        exclude = ["created_at", "updated_at"]


class MedicationKnowledgeDefinitionalDrugCharacteristicSerializer(
    WritableNestedModelSerializer
):
    """medication knowledge definitional ingredient serializer."""

    type = CodeableConceptSerializer(many=False, required=False)
    value_codeable_concept = CodeableConceptSerializer(many=False, required=False)
    value_quantity = MoneySerializer(many=False, required=False)
    value_attachment = AttachmentSerializer(many=False, required=False)

    class Meta:
        """meta options."""

        model = MedicationKnowledgeDefinitionalDrugCharacteristic
        exclude = ["created_at", "updated_at"]


class MedicationKnowledgeDefinitionalSerializer(WritableNestedModelSerializer):
    """medication knowledge definitional serializer."""

    dose_form = CodeableConceptSerializer(many=False, required=False)
    intended_route = CodeableConceptSerializer(many=False, required=False)
    ingredient = MedicationKnowledgeDefinitionalIngredientSerializer(
        required=False, many=True
    )
    drug_characteristic = MedicationKnowledgeDefinitionalDrugCharacteristicSerializer(
        many=True, required=False
    )

    class Meta:
        """meta options."""

        model = MedicationKnowledgeDefinitional
        exclude = ["created_at", "updated_at"]


class MedicationKnowledgeSerializer(BaseWritableNestedModelSerializer):
    """Medication knowledge serializer."""

    identifier = IdentifierSerializer(many=True, required=False)
    code = CodeableConceptSerializer(many=False, required=False)
    author = ContactDetailSerializer(many=False, required=False)
    jurisdiction = CodeableConceptSerializer(many=True, required=False)
    related_medication_knowledge = (
        MedicationKnowledgeRelatedMedicationKnowledgeSerializer(
            many=True, required=False
        )
    )
    associate_medication = MedicationReferenceSerializer(many=True, required=False)
    product_type = CodeableConceptSerializer(many=True, required=False)
    monograph = MedicationKnowledgeMonographSerializer(many=True, required=False)
    cost = MedicationKnowledgeCostSerializer(many=True, required=False)
    monitoring_program = MedicationKnowledgeMonitoringProgramSerializer(
        many=True, required=False
    )
    indication_guideline = MedicationKnowledgeIndicationGuidelineSerializer(
        many=True, required=False
    )
    medicine_classification = MedicationKnowledgeMedicineClassificationSerializer(
        many=True, required=False
    )
    packaging = MedicationKnowledgePackagingSerializer(many=True, required=False)
    storage_guideline = MedicationKnowledgeStorageGuidelineSerializer(
        many=True, required=False
    )
    regulatory = MedicationKnowledgeRegulatorySerializer(many=True, required=False)
    definitional = MedicationKnowledgeDefinitionalSerializer(many=False, required=False)

    class Meta:
        """Meta class."""

        model = MedicationKnowledge
        exclude = ["created_at", "updated_at"]



"""Enrollment request choices."""

from django.db import models
from django.utils.translation import gettext_lazy as _


class EnrollmentRequestStatus(models.TextChoices):
    """Enrollment Request Status Choices."""

    ACTIVE = "active", _("Active")
    CANCELLED = "cancelled", _("Cancelled")
    DRAFT = "draft", _("Draft")
    ENTERED_IN_ERROR = "entered-in-error", _("Entered in Error")



# Generated by Django 5.2 on 2025-04-23 12:46

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('base', '0005_initial'),
        ('groups', '0001_initial'),
        ('organizations', '0001_initial'),
        ('patients', '0001_initial'),
        ('practitioners', '0001_initial'),
        ('relatedpersons', '0001_initial'),
    ]

    operations = [
        migrations.AddField(
            model_name='patient',
            name='general_practitioner',
            field=models.ManyToManyField(blank=True, related_name='patient_general_practitioner', to='practitioners.practitionerorganizationpractitionerrolereference'),
        ),
        migrations.AddField(
            model_name='patient',
            name='identifier',
            field=models.ManyToManyField(blank=True, related_name='patient_identifier', to='base.identifier'),
        ),
        migrations.AddField(
            model_name='patient',
            name='managing_organization',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='patient_managing_organization', to='base.organizationreference'),
        ),
        migrations.AddField(
            model_name='patient',
            name='marital_status',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='patient_marital_status', to='base.codeableconcept'),
        ),
        migrations.AddField(
            model_name='patient',
            name='name',
            field=models.ManyToManyField(blank=True, related_name='patient_name', to='base.humanname'),
        ),
        migrations.AddField(
            model_name='patient',
            name='photo',
            field=models.ManyToManyField(blank=True, related_name='patient_photo', to='base.attachment'),
        ),
        migrations.AddField(
            model_name='patient',
            name='telecom',
            field=models.ManyToManyField(blank=True, related_name='patient_telecom', to='base.contactpoint'),
        ),
        migrations.AddField(
            model_name='patientcontact',
            name='address',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='patient_contact_address', to='base.address'),
        ),
        migrations.AddField(
            model_name='patientcontact',
            name='name',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='patient_contact_name', to='base.humanname'),
        ),
        migrations.AddField(
            model_name='patientcontact',
            name='organization',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='patient_contact_organization', to='base.organizationreference'),
        ),
        migrations.AddField(
            model_name='patientcontact',
            name='period',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='patient_contact_period', to='base.period'),
        ),
        migrations.AddField(
            model_name='patientcontact',
            name='relationship',
            field=models.ManyToManyField(blank=True, related_name='patient_contact_relationship', to='base.codeableconcept'),
        ),
        migrations.AddField(
            model_name='patientcontact',
            name='telecom',
            field=models.ManyToManyField(blank=True, related_name='patient_contact_telecom', to='base.contactpoint'),
        ),
        migrations.AddField(
            model_name='patient',
            name='contact',
            field=models.ManyToManyField(blank=True, related_name='patient_contact', to='patients.patientcontact'),
        ),
        migrations.AddField(
            model_name='patientgroupreference',
            name='group',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='patient_group_reference_group', to='groups.group'),
        ),
        migrations.AddField(
            model_name='patientgroupreference',
            name='identifier',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='patient_group_reference_identifier', to='base.identifier'),
        ),
        migrations.AddField(
            model_name='patientgroupreference',
            name='patient',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='patient_group_reference_patient', to='patients.patient'),
        ),
        migrations.AddField(
            model_name='patientorganizationreference',
            name='identifier',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='patient_organization_reference_identifier', to='base.identifier'),
        ),
        migrations.AddField(
            model_name='patientorganizationreference',
            name='organization',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='patient_organization_reference_organization', to='organizations.organization'),
        ),
        migrations.AddField(
            model_name='patientorganizationreference',
            name='patient',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='patient_organization_reference_patient', to='patients.patient'),
        ),
        migrations.AddField(
            model_name='patientpractitionerreference',
            name='identifier',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='patient_practitioner_reference_identifier', to='base.identifier'),
        ),
        migrations.AddField(
            model_name='patientpractitionerreference',
            name='patient',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='patient_practitioner_reference_patient', to='patients.patient'),
        ),
        migrations.AddField(
            model_name='patientpractitionerreference',
            name='practitioner',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='patient_practitioner_reference_practitioner', to='practitioners.practitioner'),
        ),
        migrations.AddField(
            model_name='patientreference',
            name='identifier',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='patient_reference_identifier', to='base.identifier'),
        ),
        migrations.AddField(
            model_name='patientreference',
            name='patient',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='patient_reference_patient', to='patients.patient'),
        ),
        migrations.AddField(
            model_name='patientrelatedpersonreference',
            name='identifier',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='patient_related_person_reference_identifier', to='base.identifier'),
        ),
        migrations.AddField(
            model_name='patientrelatedpersonreference',
            name='patient',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='patient_related_person_reference_patient', to='patients.patient'),
        ),
        migrations.AddField(
            model_name='patientrelatedpersonreference',
            name='related_person',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='patient_related_person_reference_related_person', to='relatedpersons.relatedperson'),
        ),
        migrations.AddField(
            model_name='patientlink',
            name='other',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='patient_link_other', to='patients.patientrelatedpersonreference'),
        ),
    ]



"""supply requests urls."""

from django.urls import path

from dfhir.supplyrequests import views

app_name = "supplyrequests"

urlpatterns = [
    path("supplyrequests/", views.SupplyRequestListView.as_view(), name="list"),
    path(
        "supplyrequests/<int:pk>/",
        views.SupplyRequestDetailView.as_view(),
        name="detail",
    ),
]



"""service requests serializers."""

from drf_writable_nested.serializers import WritableNestedModelSerializer
from rest_framework import serializers

from dfhir.activitydefinitions.serializers import (
    ActiviyDefinitionPlanDefinitionReferenceSerializer,
)
from dfhir.base.serializers import (
    AnnotationSerializer,
    BaseReferenceModelSerializer,
    CodeableConceptSerializer,
    CodeableReferenceSerializer,
    IdentifierSerializer,
    PeriodSerializer,
    QuantitySerializer,
    RangeSerializer,
    RatioSerializer,
    ReferenceSerializer,
    TimingSerializer,
)
from dfhir.base.validators import validate_date_time_fields
from dfhir.bodystructures.serializers import BodyStructureReferenceSerializer
from dfhir.coverages.serializers import CoverageClaimResponseReferenceSerializer
from dfhir.documentreferences.serializers import DocumentReferenceSerializer
from dfhir.encounters.serializers import EncounterReferenceSerializer
from dfhir.locations.serializers import LocationCodeableReferenceSerializer
from dfhir.provenances.serializers import ProvenanceReferenceSerializer

from .models import (
    AsNeeded,
    BodySite,
    OrderDetailParameter,
    OrderDetailParameterFocusCodeableReference,
    OrderDetailParameterFocusReference,
    Parameter,
    ProcedureCodes,
    ProcedureReason,
    ServiceRequest,
    ServiceRequestBasedOnReference,
    ServiceRequestCategory,
    ServiceRequestOrderDetail,
    ServiceRequestPatientInstruction,
    ServiceRequestPerformerReference,
    ServiceRequestPlanDefinitionReference,
    ServiceRequestPlanDefinitionReferenceCodeableReference,
    ServiceRequestReasonCodeableReference,
    ServiceRequestReasonReference,
    ServiceRequestReference,
    ServiceRequestRequesterReference,
    ServiceRequestSubjectReference,
)


class ServiceRequestPlanDefinitionReferenceSerializer(BaseReferenceModelSerializer):
    """service request plan definition reference serializer."""

    identifier = IdentifierSerializer(many=False, required=False)

    class Meta:
        """meta options."""

        model = ServiceRequestPlanDefinitionReference
        exclude = ["created_at", "updated_at"]


class ServiceRequestPlanDefinitionReferenceCodeableReferenceSerializer(
    WritableNestedModelSerializer
):
    """service request plan definition reference codeable reference serializer."""

    concept = CodeableConceptSerializer(many=False, required=False)
    reference = ServiceRequestPlanDefinitionReferenceSerializer(
        many=False, required=False
    )

    class Meta:
        """meta options."""

        model = ServiceRequestPlanDefinitionReferenceCodeableReference
        exclude = ["created_at", "updated_at"]


class ServiceRequestBasedOnReferenceSerializer(BaseReferenceModelSerializer):
    """service request based on reference serializer."""

    identifier = IdentifierSerializer(many=False, required=False)

    class Meta:
        """meta options."""

        model = ServiceRequestBasedOnReference
        exclude = ["created_at", "updated_at"]


class OrderDetailParameterFocusReferenceSerializer(BaseReferenceModelSerializer):
    """order detail parameter focus reference serializer."""

    identifier = IdentifierSerializer(many=False, required=False)

    class Meta:
        """meta options."""

        model = OrderDetailParameterFocusReference
        exclude = ["created_at", "updated_at"]


class OrderDetailParameterFocusCodeableReferenceSerializer(
    WritableNestedModelSerializer
):
    """order detail parameter focus codeable reference serializer."""

    concept = CodeableConceptSerializer(many=False, required=False)
    reference = OrderDetailParameterFocusReferenceSerializer(many=False, required=False)

    class Meta:
        """meta options."""

        model = OrderDetailParameterFocusCodeableReference
        exclude = ["created_at", "updated_at"]


class OrderDetailParameterSerializer(WritableNestedModelSerializer):
    """order detail parameter serializer."""

    code = CodeableConceptSerializer(many=False, required=False)
    value_quantity = QuantitySerializer(many=False, required=False)
    value_ratio = RatioSerializer(many=False, required=False)
    value_range = RangeSerializer(many=False, required=False)
    value_codeable_concept = CodeableConceptSerializer(many=False, required=False)
    value_period = PeriodSerializer(many=False, required=False)
    focus = OrderDetailParameterFocusCodeableReferenceSerializer(
        many=False, required=False
    )

    class Meta:
        """meta options."""

        model = OrderDetailParameter
        exclude = ["created_at", "updated_at"]


class ServiceRequestOrderDetailSerializer(WritableNestedModelSerializer):
    """service request order detail serializer."""

    parameter_focus = OrderDetailParameterFocusCodeableReferenceSerializer(
        many=False, required=False
    )
    parameter = OrderDetailParameterSerializer(many=True, required=False)

    class Meta:
        """meta options."""

        model = ServiceRequestOrderDetail
        exclude = ["created_at", "updated_at"]


class ServiceRequestPatientInstructionSerializer(WritableNestedModelSerializer):
    """service request patient instruction serializer."""

    instruction_reference = DocumentReferenceSerializer(required=False)

    class Meta:
        """meta options."""

        model = ServiceRequestPatientInstruction
        exclude = ["created_at", "updated_at"]


class ServiceRequestSubjectReferenceSerializer(BaseReferenceModelSerializer):
    """service request subject reference serializer."""

    identifier = IdentifierSerializer(many=False, required=False)

    class Meta:
        """meta options."""

        model = ServiceRequestSubjectReference
        exclude = ["created_at", "updated_at"]


class ServiceRequestRequesterReferenceSerializer(BaseReferenceModelSerializer):
    """service request requester reference serializer."""

    identifier = IdentifierSerializer(many=False, required=False)

    class Meta:
        """meta options."""

        model = ServiceRequestRequesterReference
        exclude = ["created_at", "updated_at"]


class ServiceRequestPerformerReferenceSerializer(BaseReferenceModelSerializer):
    """service request performer reference serializer."""

    identifier = IdentifierSerializer(many=False, required=False)

    class Meta:
        """meta options."""

        model = ServiceRequestPerformerReference
        exclude = ["created_at", "updated_at"]


class ServiceRequestReasonReferenceSerializer(BaseReferenceModelSerializer):
    """service request reason reference serializer."""

    identifier = IdentifierSerializer(many=False, required=False)

    class Meta:
        """meta options."""

        model = ServiceRequestReasonReference
        exclude = ["created_at", "updated_at"]


class ServiceRequestReasonCodeableReferenceSerializer(WritableNestedModelSerializer):
    """service request reason codeable reference serializer."""

    concept = CodeableConceptSerializer(many=False, required=False)
    reference = ServiceRequestReasonReferenceSerializer(many=False, required=False)

    class Meta:
        """meta options."""

        model = ServiceRequestReasonCodeableReference
        exclude = ["created_at", "updated_at"]


class ServiceRequestReferenceSerializer(BaseReferenceModelSerializer):
    """service request reference serializer."""

    identifier = IdentifierSerializer(many=False, required=False)

    class Meta:
        """meta options."""

        model = ServiceRequestReference
        exclude = ["created_at", "updated_at"]


class ServiceRequestSerializer(WritableNestedModelSerializer):
    """service request serializer."""

    def get_fields(self):
        """Get fields."""
        from dfhir.specimens.serializers import SpecimenReferenceSerializer

        fields = super(ServiceRequestSerializer, self).get_fields()
        fields["specimen"] = SpecimenReferenceSerializer(many=True, required=False)

        return fields

    identifier = IdentifierSerializer(many=True, required=False)
    based_on = ServiceRequestBasedOnReferenceSerializer(many=True, required=False)
    replace = ServiceRequestReferenceSerializer(many=True, required=False)
    requisition = IdentifierSerializer(many=False, required=False)
    category = CodeableConceptSerializer(many=True, required=False)
    code = ActiviyDefinitionPlanDefinitionReferenceSerializer(
        many=False, required=False
    )
    order_detail = ServiceRequestOrderDetailSerializer(many=True, required=False)
    quantity_quantity = QuantitySerializer(many=False, required=False)
    quantity_ratio = RatioSerializer(many=False, required=False)
    quantity_range = RangeSerializer(many=False, required=False)
    subject = ServiceRequestSubjectReferenceSerializer(many=False, required=False)
    focus = ReferenceSerializer(many=True, required=False)
    encounter = EncounterReferenceSerializer(many=False, required=False)
    occurrence_period = PeriodSerializer(many=False, required=False)
    occurrence_timing = TimingSerializer(many=False, required=False)
    as_needed_for = CodeableConceptSerializer(many=True, required=False)
    requester = ServiceRequestRequesterReferenceSerializer(many=False, required=False)
    additional_recipient = ServiceRequestRequesterReferenceSerializer(
        many=True, required=False
    )
    performer_type = CodeableConceptSerializer(many=False, required=False)
    performer = ServiceRequestPerformerReferenceSerializer(many=True, required=False)
    location = LocationCodeableReferenceSerializer(many=True, required=False)
    reason = ServiceRequestReasonCodeableReferenceSerializer(many=True, required=False)
    insurance = CoverageClaimResponseReferenceSerializer(required=False, many=True)
    supporting_info = CodeableReferenceSerializer(many=True, required=False)
    body_site = CodeableConceptSerializer(many=True, required=False)
    body_structure = BodyStructureReferenceSerializer(many=False, required=False)
    note = AnnotationSerializer(many=True, required=False)
    patient_instruction = ServiceRequestPatientInstructionSerializer(
        many=True, required=False
    )
    relevant_history = ProvenanceReferenceSerializer(many=True, required=False)

    class Meta:
        """meta options."""

        model = ServiceRequest
        exclude = ["created_at", "updated_at"]

    def validate(self, data):
        """Service request serializer data validators."""
        validate_date_time_fields(
            data.get("start_date_time"), data.get("end_date_time")
        )
        return data


class ParameterSerializer(serializers.ModelSerializer):
    """parameter serializer class."""

    class Meta:
        """meta options."""

        model = Parameter
        exclude = ["created_at", "updated_at"]


class ServiceRequestCategorySerializer(serializers.ModelSerializer):
    """service request serializer."""

    class Meta:
        """meta options."""

        model = ServiceRequestCategory
        exclude = ["created_at", "updated_at"]


class AsNeededSerializer(serializers.ModelSerializer):
    """as needed serializer."""

    class Meta:
        """meta options."""

        model = AsNeeded
        exclude = ["created_at", "updated_at"]


class ProcedureReasonSerializer(serializers.ModelSerializer):
    """procedure reason serializer."""

    class Meta:
        """meta options."""

        model = ProcedureReason
        exclude = ["created_at", "updated_at"]


class BodySiteSerializer(serializers.ModelSerializer):
    """body site serializer."""

    class Meta:
        """meta options."""

        model = BodySite
        exclude = ["created_at", "updated_at"]


class ProcedureCodeSerializer(serializers.ModelSerializer):
    """procedure code serializer."""

    class Meta:
        """meta options."""

        model = ProcedureCodes
        exclude = ["created_at", "updated_at"]



"""medications serializers."""

from drf_writable_nested.serializers import WritableNestedModelSerializer
from rest_framework import serializers

from dfhir.base.serializers import (
    BaseReferenceModelSerializer,
    BaseWritableNestedModelSerializer,
    CodeableConceptSerializer,
    IdentifierSerializer,
    OrganizationReferenceSerializer,
    QuantitySerializer,
    RatioSerializer,
)
from dfhir.medications.models import (
    Medication,
    MedicationBatch,
    MedicationCodeableReference,
    MedicationCodes,
    MedicationDoseForm,
    MedicationIngredient,
    MedicationIngredientItem,
    MedicationReference,
    MedicationSubstanceCodeableReference,
    MedicationSubstanceReference,
)


class MedicationBatchSerializer(WritableNestedModelSerializer):
    """medication batch serializer."""

    class Meta:
        """metadata."""

        model = MedicationBatch
        exclude = ["created_at", "updated_at"]


class MedicationSubstanceReferenceSerializer(BaseReferenceModelSerializer):
    """medication substance reference serializer."""

    identifier = IdentifierSerializer(required=False, many=False)

    class Meta:
        """metadata."""

        model = MedicationSubstanceReference
        exclude = ["created_at", "updated_at"]


class MedicatonSubstanceCodeableReferenceSerializer(WritableNestedModelSerializer):
    """medication substance codeable reference serializer."""

    reference = MedicationSubstanceReferenceSerializer(required=False, many=False)
    concept = CodeableConceptSerializer(required=False, many=False)

    class Meta:
        """metadata."""

        model = MedicationSubstanceCodeableReference
        exclude = ["created_at", "updated_at"]


class MedicationIngredientSerializer(WritableNestedModelSerializer):
    """medication ingredient serializer."""

    item = MedicatonSubstanceCodeableReferenceSerializer(required=False, many=False)
    strength_quantity = QuantitySerializer(required=False, many=False)
    strength_ratio = RatioSerializer(required=False, many=False)
    strength_codeable_concept = CodeableConceptSerializer(required=False, many=False)

    class Meta:
        """metadata."""

        model = MedicationIngredient
        exclude = ["created_at", "updated_at"]


class MedicationSerializer(BaseWritableNestedModelSerializer):
    """medication serializer."""

    def get_fields(self):
        """Get fields."""
        from dfhir.medicationknowledges.serializers import (
            MedicationKnowledgeReferenceSerializer,
        )

        fields = super().get_fields()

        fields["definition"] = MedicationKnowledgeReferenceSerializer(
            required=False, many=False
        )

        return fields

    code = CodeableConceptSerializer(required=False, many=False)
    dose_form = CodeableConceptSerializer(required=False, many=False)
    batch = MedicationBatchSerializer(required=False, many=False)
    ingredient = MedicationIngredientSerializer(required=False, many=True)
    manufacturer = OrganizationReferenceSerializer(required=False, many=False)
    marketing_authorization_holder = OrganizationReferenceSerializer(
        required=False, many=False
    )

    class Meta:
        """metadata."""

        model = Medication
        exclude = ["created_at", "updated_at"]


class MedicationCodesSerializer(serializers.ModelSerializer):
    """medication codes serializer."""

    class Meta:
        """metadata."""

        model = MedicationCodes
        exclude = ["created_at", "updated_at"]


class MedicationDoseFormSerializer(serializers.ModelSerializer):
    """medication dose form serializer."""

    class Meta:
        """metadata."""

        model = MedicationDoseForm
        exclude = ["created_at", "updated_at"]


class MedicationIngredientItemSerializer(serializers.ModelSerializer):
    """medication ingredient item serializer."""

    class Meta:
        """metadata."""

        model = MedicationIngredientItem
        exclude = ["created_at", "updated_at"]


class MedicationReferenceSerializer(BaseReferenceModelSerializer):
    """medication reference serializer."""

    identifier = IdentifierSerializer(required=False, many=False)

    class Meta:
        """metadata."""

        model = MedicationReference
        exclude = ["created_at", "updated_at"]


class MedicationCodeableReferenceSerializer(WritableNestedModelSerializer):
    """medication codeable reference serializer."""

    concept = CodeableConceptSerializer(required=False, many=False)
    reference = MedicationReferenceSerializer(required=False, many=False)

    class Meta:
        """metadata."""

        model = MedicationCodeableReference
        exclude = ["created_at", "updated_at"]



"""payment reconciliations models."""

from django.db import models

from dfhir.base.models import (
    BaseReference,
    CodeableConcept,
    Identifier,
    TimeStampedModel,
)
from dfhir.paymentreconciliations.choices import (
    PaymentReconciliationOutcomeChoices,
    PaymentReconciliationProcessNoteChoices,
    PaymentReconciliationStatusChoices,
)


class OrganizationPractitionerPractitionerRoleReference(BaseReference):
    """payment reconciliations payment issuer reference model."""

    identifier = models.ForeignKey(
        Identifier,
        on_delete=models.CASCADE,
        null=True,
        related_name="organization_practitioner_practitioner_role_reference_identifier",
    )

    organization = models.ForeignKey(
        "organizations.Organization",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="organization_practitioner_practitioner_role_reference_organization",
    )
    patient = models.ForeignKey(
        "patients.PatientReference",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="organization_practitioner_practitioner_role_reference_patient",
    )
    related_person = models.ForeignKey(
        "relatedpersons.RelatedPerson",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="organization_practitioner_practitioner_role_reference_related_person",
    )


class PaymentReconciliationAllocationTargetReference(BaseReference):
    """payment reconciliations allocation target reference model."""

    identifier = models.ForeignKey(
        Identifier,
        on_delete=models.CASCADE,
        null=True,
        related_name="payment_reconciliation_allocation_target_reference_identifier",
    )
    claim = models.ForeignKey(
        "claims.Claim",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="payment_reconciliation_allocation_target_reference_claim",
    )
    account = models.ForeignKey(
        "accounts.Account",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="payment_reconciliation_allocation_target_reference_account",
    )
    invoice = models.ForeignKey(
        "invoices.Invoice",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="payment_reconciliation_allocation_target_reference_invoice",
    )
    charge_item = models.ForeignKey(
        "chargeitems.ChargeItem",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="payment_reconciliation_allocation_target_reference_charge_item",
    )
    encounter = models.ForeignKey(
        "encounters.Encounter",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="payment_reconciliation_allocation_target_reference_encounter",
    )
    contract = models.ForeignKey(
        "contracts.Contract",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="payment_reconciliation_allocation_target_reference_contract",
    )


class PaymentReconciliationPaymentIssuerReference(BaseReference):
    """payment reconciliations payment issuer reference model."""

    identifier = models.ForeignKey(
        Identifier,
        on_delete=models.CASCADE,
        null=True,
        related_name="payment_reconciliation_payment_issuer_reference_identifier",
    )
    organization = models.ForeignKey(
        "organizations.Organization",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="payment_reconciliation_payment_issuer_reference_organization",
    )
    patient = models.ForeignKey(
        "patients.PatientReference",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="payment_reconciliation_payment_issuer_reference_patient",
    )
    related_person = models.ForeignKey(
        "relatedpersons.RelatedPerson",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="payment_reconciliation_payment_issuer_reference_related_person",
    )


class PaymentReconciliationAllocation(TimeStampedModel):
    """payment reconciliations allocation model."""

    identifier = models.ForeignKey(
        Identifier,
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="payment_reconciliation_allocation_identifier",
    )
    predecessor = models.ForeignKey(
        Identifier,
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="payment_reconciliation_allocation_predecessor",
    )
    target = models.ForeignKey(
        PaymentReconciliationAllocationTargetReference,
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="payment_reconciliation_allocation_target",
    )
    target_item_string = models.CharField(max_length=255, null=True)
    target_item_identifier = models.ForeignKey(
        Identifier,
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="payment_reconciliation_allocation_target_item_identifier",
    )
    target_item_positive_int = models.PositiveIntegerField(null=True)
    encounter = models.ForeignKey(
        "encounters.EncounterReference",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="payment_reconciliation_allocation_encounter",
    )
    account = models.ForeignKey(
        "accounts.AccountReference",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="payment_reconciliation_allocation_account",
    )
    type = models.ForeignKey(
        "base.CodeableConcept",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="payment_reconciliation_allocation_type",
    )
    submitter = models.ForeignKey(
        OrganizationPractitionerPractitionerRoleReference,
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="payment_reconciliation_allocation_submitter",
    )
    response = models.ForeignKey(
        "claimresponses.ClaimResponse",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="payment_reconciliation_allocation_response",
    )
    date = models.DateField(null=True)
    responsible = models.ForeignKey(
        "practitionerroles.PractitionerRoleReference",
        on_delete=models.CASCADE,
        null=True,
        related_name="payment_reconciliation_allocation_responsible",
    )
    payee = models.ForeignKey(
        OrganizationPractitionerPractitionerRoleReference,
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="payment_reconciliation_allocation_payee",
    )
    amount = models.ForeignKey(
        "base.Money",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="payment_reconciliation_allocation_amount",
    )


class PaymentReconciliationProcessNote(TimeStampedModel):
    """payment reconciliations process note model."""

    type = models.CharField(
        max_length=255,
        null=True,
        choices=PaymentReconciliationProcessNoteChoices.choices,
    )
    text = models.CharField(max_length=255, null=True)


class PaymentReconciliation(TimeStampedModel):
    """payment reconciliations model."""

    identifier = models.ManyToManyField(
        Identifier, related_name="payment_reconciliation_identifier", blank=True
    )
    type = models.ForeignKey(
        CodeableConcept,
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="payment_reconciliation_type",
    )
    status = models.CharField(
        max_length=255, null=True, choices=PaymentReconciliationStatusChoices.choices
    )
    kind = models.ForeignKey(
        CodeableConcept,
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="payment_reconciliation_kind",
    )
    period = models.ForeignKey(
        "base.Period",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="payment_reconciliation_period",
    )
    created = models.DateTimeField(null=True)
    enterer = models.ForeignKey(
        OrganizationPractitionerPractitionerRoleReference,
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="payment_reconciliation_enterer",
    )
    issuer_type = models.ForeignKey(
        CodeableConcept,
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="payment_reconciliation_issuer_type",
    )
    payment_issuer = models.ForeignKey(
        PaymentReconciliationPaymentIssuerReference,
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="payment_reconciliation_payment_issuer",
    )
    request = models.ForeignKey(
        "tasks.TaskReference",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="payment_reconciliation_request",
    )
    requestor = models.ForeignKey(  # codespell:ignore requestor
        OrganizationPractitionerPractitionerRoleReference,
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="payment_reconciliation_requestor",
    )
    outcome = models.CharField(
        max_length=255, null=True, choices=PaymentReconciliationOutcomeChoices.choices
    )
    disposition = models.CharField(max_length=255, null=True)
    date = models.DateField(null=True)
    location = models.ForeignKey(
        "locations.LocationReference",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="payment_reconciliation_location",
    )
    method = models.ForeignKey(
        CodeableConcept,
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="payment_reconciliation_method",
    )
    card_brand = models.CharField(max_length=255, null=True)
    account_number = models.CharField(max_length=255, null=True)
    expiration_date = models.DateField(null=True)
    processor = models.CharField(max_length=255, null=True)
    reference_number = models.CharField(max_length=255, null=True)
    authorization = models.DateField(null=True)
    tendered_amount = models.ForeignKey(
        "base.Money",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="payment_reconciliation_tendered_amount",
    )
    returned_amount = models.ForeignKey(
        "base.Money",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="payment_reconciliation_returned_amount",
    )
    amount = models.ForeignKey(
        "base.Money",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="payment_reconciliation_amount",
    )
    payment_identifier = models.ForeignKey(
        Identifier,
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="payment_reconciliation_payment_identifier",
    )
    allocation = models.ManyToManyField(
        PaymentReconciliationAllocation,
        related_name="payment_reconciliation_allocation",
        blank=True,
    )
    form_code = models.ForeignKey(
        CodeableConcept,
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="payment_reconciliation_form_code",
    )
    process_note = models.ManyToManyField(
        PaymentReconciliationProcessNote,
        related_name="payment_reconciliation_process_note",
        blank=True,
    )


class PaymentReconciliationReference(BaseReference):
    """payment reconciliations reference model."""

    identifier = models.ForeignKey(
        Identifier,
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="payment_reconciliation_reference_identifier",
    )
    payment_reconciliation = models.ForeignKey(
        PaymentReconciliation,
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="payment_reconciliation_reference_payment_reconciliation",
    )



"""nutrition intake urls."""

from django.urls import path

from dfhir.nutritionintakes.views import (
    NutritionIntakeDetailView,
    NutritionIntakeListView,
)

app_name = "nutritionintakes"


urlpatterns = [
    path("nutritionintakes/", NutritionIntakeListView.as_view(), name="list"),
    path(
        "nutritionintakes/<int:pk>/", NutritionIntakeDetailView.as_view(), name="detail"
    ),
]



# Generated by Django 5.2 on 2025-04-23 12:46

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('base', '0001_initial'),
        ('endpoints', '0001_initial'),
        ('healthcareservices', '0001_initial'),
        ('locations', '0001_initial'),
    ]

    operations = [
        migrations.CreateModel(
            name='OrganizationAffiliation',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('active', models.BooleanField(default=True)),
                ('code', models.ManyToManyField(blank=True, related_name='organization_affiliation_code', to='base.codeableconcept')),
                ('contact', models.ManyToManyField(blank=True, related_name='organization_affiliation_contact', to='base.extendedcontactdetail')),
                ('endpoint', models.ManyToManyField(blank=True, related_name='organization_affiliation_endpoint', to='endpoints.endpointreference')),
                ('healthcare_service', models.ManyToManyField(blank=True, related_name='organization_affiliation_healthcare_service', to='healthcareservices.healthcareservicereference')),
                ('identifier', models.ManyToManyField(blank=True, related_name='organization_affiliations', to='base.identifier')),
                ('location', models.ManyToManyField(blank=True, related_name='organization_affiliation_location', to='locations.locationreference')),
                ('network', models.ManyToManyField(blank=True, related_name='network_organization_affiliations', to='base.organizationreference')),
                ('organization', models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='organization_affiliations', to='base.organizationreference')),
                ('participating_organization', models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='participating_organization_affiliations', to='base.organizationreference')),
                ('period', models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='organization_affiliations_period', to='base.period')),
                ('specialty', models.ManyToManyField(blank=True, related_name='organization_affiliation_specialty', to='base.codeableconcept')),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='OrganizationAffiliationReference',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('reference', models.CharField(max_length=255, null=True)),
                ('type', models.CharField(max_length=255, null=True)),
                ('display', models.CharField(max_length=255, null=True)),
                ('identifier', models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='organization_affiliation_references_identifier', to='base.identifier')),
                ('organization_affiliation', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='organization_affiliation_references_organization_affiliation', to='organizationaffiliations.organizationaffiliation')),
            ],
            options={
                'abstract': False,
            },
        ),
    ]



"""Biologically Derived Products models."""

from django.db import models

from dfhir.base.models import (
    Attachment,
    BaseReference,
    CodeableConcept,
    Coding,
    Identifier,
    OrganizationReference,
    Period,
    Quantity,
    Range,
    Ratio,
    TimeStampedModel,
)
from dfhir.patients.models import PatientOrganizationReference
from dfhir.practitioners.models import PractitionerPractitionerRoleReference


class BiologicallyDerivedProductReference(BaseReference):
    """Biologically Derived Product Reference model."""

    identifier = models.ForeignKey(
        Identifier,
        on_delete=models.CASCADE,
        related_name="biologically_derived_product_reference_identifier",
        null=True,
    )
    biologically_derived_product = models.ForeignKey(
        "BiologicallyDerivedProduct",
        on_delete=models.CASCADE,
        related_name="biologically_derived_product_reference_biologically_derived_product",
        null=True,
    )


class BiologicallyDerivedProductCollection(TimeStampedModel):
    """Biologically Derived Product Collection model."""

    collector = models.ForeignKey(
        PractitionerPractitionerRoleReference,
        related_name="biologically_derived_product_collections_collector",
        on_delete=models.CASCADE,
        null=True,
    )
    source = models.ForeignKey(
        PatientOrganizationReference,
        related_name="biologically_derived_product_collections_source",
        on_delete=models.CASCADE,
        null=True,
    )
    collected_date_time = models.DateTimeField(null=True)
    collected_period = models.ForeignKey(
        Period,
        related_name="biologically_derived_product_collections_collected_period",
        on_delete=models.CASCADE,
        null=True,
    )
    procedure = models.ForeignKey(
        "procedures.Procedure",
        related_name="biologically_derived_product_collections_procedure",
        on_delete=models.CASCADE,
        null=True,
    )


class BiologicallyDerivedProductProperty(TimeStampedModel):
    """Biologically Derived Product Property model."""

    type = models.ForeignKey(
        CodeableConcept,
        related_name="biologically_derived_product_properties_type",
        on_delete=models.CASCADE,
        null=True,
    )
    value_boolean = models.BooleanField(null=True)
    value_integer = models.IntegerField(null=True)
    value_codeable_concept = models.ForeignKey(
        CodeableConcept,
        related_name="biologically_derived_product_properties_value_codeable_concept",
        on_delete=models.CASCADE,
        null=True,
    )
    value_period = models.ForeignKey(
        Period,
        related_name="biologically_derived_product_properties_value_period",
        on_delete=models.CASCADE,
        null=True,
    )
    value_quantity = models.ForeignKey(
        Quantity,
        related_name="biologically_derived_product_properties_value_quantity",
        on_delete=models.CASCADE,
        null=True,
    )
    value_range = models.ForeignKey(
        Range,
        related_name="biologically_derived_product_properties_value_range",
        on_delete=models.CASCADE,
        null=True,
    )
    value_ratio = models.ForeignKey(
        Ratio,
        related_name="biologically_derived_product_properties_value_ratio",
        on_delete=models.CASCADE,
        null=True,
    )
    value_string = models.CharField(max_length=255, null=True)
    value_attachment = models.ForeignKey(
        Attachment,
        related_name="biologically_derived_product_properties_value_attachment",
        on_delete=models.CASCADE,
        null=True,
    )


class BiologicallyDerivedProduct(TimeStampedModel):
    """Biologically Derived Product model."""

    product_category = models.ManyToManyField(
        CodeableConcept,
        related_name="biologically_derived_products_product_category",
        blank=True,
    )
    product_code = models.ForeignKey(
        CodeableConcept,
        related_name="biologically_derived_products_product_code",
        on_delete=models.CASCADE,
        null=True,
    )
    parent = models.ManyToManyField(
        BiologicallyDerivedProductReference,
        related_name="biologically_derived_products_parent",
        blank=True,
    )
    request = models.ManyToManyField(
        "servicerequests.ServiceRequestReference",
        related_name="biologically_derived_products_request",
        blank=True,
    )
    identifier = models.ManyToManyField(
        Identifier, related_name="biologically_derived_products_identifier", blank=True
    )
    biological_source_event = models.ForeignKey(
        Identifier,
        related_name="biologically_derived_products_biological_source_event",
        on_delete=models.CASCADE,
        null=True,
    )
    processing_facility = models.ManyToManyField(
        OrganizationReference,
        related_name="biologically_derived_products_processing_faciality",
        blank=True,
    )
    division = models.CharField(max_length=255, null=True)
    product_status = models.ForeignKey(
        Coding,
        related_name="biologically_derived_products_product_status",
        on_delete=models.CASCADE,
        null=True,
    )
    expiration_date = models.DateTimeField(null=True)
    collection = models.ForeignKey(
        BiologicallyDerivedProductCollection,
        related_name="biologically_derived_products_collection",
        on_delete=models.CASCADE,
        null=True,
    )
    storage_temp_requirements = models.ForeignKey(
        Range,
        related_name="biologically_derived_products_storage_temp_requirements",
        on_delete=models.CASCADE,
        null=True,
    )
    property = models.ManyToManyField(
        BiologicallyDerivedProductProperty,
        related_name="biologically_derived_products_property",
        blank=True,
    )



# Generated by Django 5.2 on 2025-04-23 12:46

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('base', '0005_initial'),
        ('biologicallyderivedproducts', '0001_initial'),
        ('detectedissues', '0003_initial'),
        ('devices', '0001_initial'),
        ('groups', '0001_initial'),
        ('locations', '0001_initial'),
        ('medications', '0001_initial'),
        ('nutritionproducts', '0001_initial'),
        ('organizations', '0001_initial'),
        ('patients', '0002_initial'),
        ('practitionerroles', '0002_initial'),
        ('practitioners', '0001_initial'),
        ('relatedpersons', '0001_initial'),
    ]

    operations = [
        migrations.AddField(
            model_name='detectedissueauthorreference',
            name='practitioner',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='detected_issue_author_reference_practitioner', to='practitioners.practitioner'),
        ),
        migrations.AddField(
            model_name='detectedissueauthorreference',
            name='practitioner_role',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='detected_issue_author_reference_practitioner_role', to='practitionerroles.practitionerrole'),
        ),
        migrations.AddField(
            model_name='detectedissueauthorreference',
            name='related_person',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='detected_issue_author_reference_related_person', to='relatedpersons.relatedperson'),
        ),
        migrations.AddField(
            model_name='detectedissue',
            name='author',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='detected_issue_author', to='detectedissues.detectedissueauthorreference'),
        ),
        migrations.AddField(
            model_name='detectedissuecodeablereference',
            name='concept',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='detected_issue_codeable_reference_concept', to='base.codeableconcept'),
        ),
        migrations.AddField(
            model_name='detectedissueevidence',
            name='code',
            field=models.ManyToManyField(blank=True, related_name='detected_issue_evidence_code', to='base.codeableconcept'),
        ),
        migrations.AddField(
            model_name='detectedissueevidence',
            name='detail',
            field=models.ManyToManyField(blank=True, related_name='detected_issue_evidence_detail', to='base.reference'),
        ),
        migrations.AddField(
            model_name='detectedissue',
            name='evidence',
            field=models.ManyToManyField(blank=True, related_name='detected_issue_evidence', to='detectedissues.detectedissueevidence'),
        ),
        migrations.AddField(
            model_name='detectedissuemitigation',
            name='action',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='detected_issue_mitigation_action', to='base.codeableconcept'),
        ),
        migrations.AddField(
            model_name='detectedissuemitigation',
            name='author',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='detected_issue_mitigation_author', to='practitioners.practitionerpractitionerrolereference'),
        ),
        migrations.AddField(
            model_name='detectedissuemitigation',
            name='note',
            field=models.ManyToManyField(blank=True, related_name='detected_issue_mitigation_note', to='base.annotation'),
        ),
        migrations.AddField(
            model_name='detectedissue',
            name='mitigation',
            field=models.ManyToManyField(blank=True, related_name='detected_issue_mitigation', to='detectedissues.detectedissuemitigation'),
        ),
        migrations.AddField(
            model_name='detectedissuereference',
            name='detected_issue',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='detected_issue_reference_detected_issue', to='detectedissues.detectedissue'),
        ),
        migrations.AddField(
            model_name='detectedissuereference',
            name='identifier',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='detected_issue_reference_identifier', to='base.identifier'),
        ),
        migrations.AddField(
            model_name='detectedissuecodeablereference',
            name='reference',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='detected_issue_codeable_reference_reference', to='detectedissues.detectedissuereference'),
        ),
        migrations.AddField(
            model_name='detectedissuesubjectreference',
            name='biologically_derived_product',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='detected_issue_subject_reference_biologically_derived_product', to='biologicallyderivedproducts.biologicallyderivedproduct'),
        ),
        migrations.AddField(
            model_name='detectedissuesubjectreference',
            name='device',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='detected_issue_subject_reference_device', to='devices.device'),
        ),
        migrations.AddField(
            model_name='detectedissuesubjectreference',
            name='group',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='detected_issue_subject_reference_group', to='groups.group'),
        ),
        migrations.AddField(
            model_name='detectedissuesubjectreference',
            name='identifier',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='detected_issue_subject_reference_identifier', to='base.identifier'),
        ),
        migrations.AddField(
            model_name='detectedissuesubjectreference',
            name='location',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='detected_issue_subject_reference_location', to='locations.location'),
        ),
        migrations.AddField(
            model_name='detectedissuesubjectreference',
            name='medication',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='detected_issue_subject_reference_medication', to='medications.medication'),
        ),
        migrations.AddField(
            model_name='detectedissuesubjectreference',
            name='nutrition_product',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='detected_issue_subject_reference_nutrition_product', to='nutritionproducts.nutritionproduct'),
        ),
        migrations.AddField(
            model_name='detectedissuesubjectreference',
            name='organization',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='detected_issue_subject_reference_organization', to='organizations.organization'),
        ),
        migrations.AddField(
            model_name='detectedissuesubjectreference',
            name='patient',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='detected_issue_subject_reference_patient', to='patients.patient'),
        ),
        migrations.AddField(
            model_name='detectedissuesubjectreference',
            name='practitioner',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='detected_issue_subject_reference_practitioner', to='practitioners.practitioner'),
        ),
    ]



"""EPisode of Care app configuration."""

from django.apps import AppConfig
from django.utils.translation import gettext_lazy as _


class EpisodeofcareConfig(AppConfig):
    """Episode of Care app configuration."""

    default_auto_field = "django.db.models.BigAutoField"
    name = "dfhir.episodeofcares"
    verbose_name = _("Episode of Care")



"""document reference views."""

from django.http import Http404
from drf_spectacular.utils import extend_schema
from rest_framework import status
from rest_framework.permissions import AllowAny
from rest_framework.response import Response
from rest_framework.views import APIView

from dfhir.documentreferences.models import DocumentReference
from dfhir.documentreferences.serializers import DocumentReferenceSerializer


class DocumentReferenceListView(APIView):
    """DocumentReference list view."""

    permission_classes = [AllowAny]

    @extend_schema(responses={200: DocumentReferenceSerializer(many=True)})
    def get(self, request):
        """Get document references."""
        document_references = DocumentReference.objects.all()
        serializer = DocumentReferenceSerializer(document_references, many=True)
        return Response(serializer.data)

    @extend_schema(
        request=DocumentReferenceSerializer,
        responses={201: DocumentReferenceSerializer},
    )
    def post(self, request):
        """Create a document reference."""
        serializer = DocumentReferenceSerializer(data=request.data)
        serializer.is_valid(raise_exception=True)
        serializer.save()
        return Response(serializer.data, status=status.HTTP_201_CREATED)


class DocumentReferenceDetailView(APIView):
    """DocumentReference detail view."""

    permission_classes = [AllowAny]

    def get_object(self, pk):
        """Get document reference object."""
        try:
            return DocumentReference.objects.get(pk=pk)
        except DocumentReference.DoesNotExist as err:
            raise Http404 from err

    @extend_schema(responses={200: DocumentReferenceSerializer})
    def get(self, request, pk):
        """Get a document reference."""
        document_reference = self.get_object(pk)
        serializer = DocumentReferenceSerializer(document_reference)
        return Response(serializer.data)

    @extend_schema(
        request=DocumentReferenceSerializer,
        responses={200: DocumentReferenceSerializer},
    )
    def patch(self, request, pk):
        """Update a document reference."""
        document_reference = self.get_object(pk)
        serializer = DocumentReferenceSerializer(
            document_reference, data=request.data, partial=True
        )
        serializer.is_valid(raise_exception=True)
        serializer.save()
        return Response(serializer.data, status=status.HTTP_200_OK)

    @extend_schema(responses={204: None})
    def delete(self, request, pk):
        """Delete a document reference."""
        document_reference = self.get_object(pk)
        document_reference.delete()
        return Response(status=status.HTTP_204_NO_CONTENT)



"""supply delivery models."""

from django.db import models

from dfhir.base.models import BaseReference, Identifier, TimeStampedModel
from dfhir.supplydeliveries.choices import SupplyDeliveryStatusChoices


class SupplyDeliveryContractReference(BaseReference):
    """supply delivery contract reference model."""

    identifier = models.ForeignKey(
        Identifier,
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="supply_delivery_contracts_reference_identifier",
    )
    supply_delivery = models.ForeignKey(
        "SupplyDelivery",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="supply_delivery_contracts_supply_delivery",
    )
    contract = models.ForeignKey(
        "contracts.Contract",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="supply_delivery_contracts_reference_contract",
    )


class SupplyDeliverReference(BaseReference):
    """supply delivery reference."""

    identifier = models.ForeignKey(
        Identifier,
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="supply_delivery_reference_identifier",
    )
    supply_delivery = models.ForeignKey(
        "SupplyDelivery",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="supply_delivery_reference_contract",
    )


class SupplyDeliverySuppliedItemItemReference(BaseReference):
    """supply delivery supplied item item reference."""

    identifier = models.ForeignKey(
        Identifier,
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="supply_delivery_supplied_item_item_references_identifier",
    )
    medication = models.ForeignKey(
        "medications.Medication",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="supply_delivery_supplied_item_item_references_medication",
    )
    sustance = models.ForeignKey(
        "substances.Substance",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="supply_delivery_supplied_item_item_references_substance",
    )
    device = models.ForeignKey(
        "devices.Device",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="supply_delivery_supplied_item_item_references_device",
    )
    biologically_derived_product = models.ForeignKey(
        "biologicallyderivedproducts.BiologicallyDerivedProduct",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="supply_delivery_supplied_item_item_references_biologically_derived_product",
    )
    nutrition_product = models.ForeignKey(
        "nutritionproducts.NutritionProduct",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="supply_delivery_supplied_item_item_references_nutrition_product",
    )
    inventory_item = models.ForeignKey(
        "inventoryitems.InventoryItem",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="supply_delivery_supplied_item_item_references_inventory_item",
    )


class SupplyDeliverySupplierReference(BaseReference):
    """supply delivery supplier reference."""

    identifier = models.ForeignKey(
        Identifier,
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="supply_delivery_supplier_references_identifier",
    )
    organization = models.ForeignKey(
        "organizations.Organization",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="supply_delivery_supplier_references_organization",
    )

    practitioner = models.ForeignKey(
        "practitioners.Practitioner",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="supply_delivery_supplier_references_practitioner",
    )
    practitioner_role = models.ForeignKey(
        "practitionerroles.PractitionerRole",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="supply_delivery_supplier_references_practitioner_role",
    )


class SupplyDeliveryDestinationReference(BaseReference):
    """supply delivery destination reference."""

    identifier = models.ForeignKey(
        Identifier,
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="supply_delivery_destination_references_identifier",
    )
    location = models.ForeignKey(
        "locations.Location",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="supply_delivery_destination_references_location",
    )
    practitioner = models.ForeignKey(
        "practitioners.Practitioner",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="supply_delivery_destination_references_practitioner",
    )
    practitioner_role = models.ForeignKey(
        "practitionerroles.PractitionerRole",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="supply_delivery_destination_references_practitioner_role",
    )
    organization = models.ForeignKey(
        "organizations.Organization",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="supply_delivery_destination_references_organization",
    )


class SupplyDeliveryReceiverReference(BaseReference):
    """supply delivery receiver reference model."""

    identifier = models.ForeignKey(
        Identifier,
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="supply_delivery_receiver_references_identifier",
    )
    practitioner = models.ForeignKey(
        "practitioners.Practitioner",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="supply_delivery_receiver_references_practitioner",
    )
    practitioner_role = models.ForeignKey(
        "practitionerroles.PractitionerRole",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="supply_delivery_receiver_references_practitioner_role",
    )
    organization = models.ForeignKey(
        "organizations.Organization",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="supply_delivery_receiver_references_organization",
    )


class SupplyDeliverySuppliedItem(TimeStampedModel):
    """supply delivery supplied item class."""

    quantity = models.ForeignKey(
        "base.SimpleQuantity",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="supply_delivery_supplied_item_quantity",
    )
    condition = models.ForeignKey(
        "base.CodeableConcept",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="supply_delivery_supplied_item_condition",
    )
    item_codeable_concept = models.ForeignKey(
        "base.CodeableConcept",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="supply_delivery_supplied_item_item_codealbe_concept",
    )
    item_reference = models.ForeignKey(
        SupplyDeliverySuppliedItemItemReference,
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="supply_delivery_supplied_item_item_reference",
    )


class SupplyDelivery(TimeStampedModel):
    """supply delivery model."""

    identifier = models.ManyToManyField(
        Identifier, blank=True, related_name="supply_delivery_identifier"
    )
    based_on = models.ManyToManyField(
        "supplyrequests.SupplyRequestReference",
        blank=True,
        related_name="supply_delivery_based_on",
    )
    part_of = models.ManyToManyField(
        SupplyDeliveryContractReference,
        blank=True,
        related_name="supply_delivery_part_of",
    )
    status = models.CharField(
        max_length=255, null=True, choices=SupplyDeliveryStatusChoices.choices
    )
    patient = models.ForeignKey(
        "patients.PatientReference",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="supply_delivery_patient",
    )
    type = models.ForeignKey(
        "base.CodeableConcept",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="supply_delivery_type",
    )
    stage = models.ForeignKey(
        "base.CodeableConcept",
        related_name="supply_delivery_stage",
        on_delete=models.DO_NOTHING,
        null=True,
    )

    supplied_item = models.ManyToManyField(
        SupplyDeliverySuppliedItem,
        blank=True,
        related_name="supply_delivery_supplied_item",
    )
    occurrence_date_time = models.DateTimeField(null=True, blank=True)
    occurrence_period = models.ForeignKey(
        "base.Period",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="supply_delivery_occurrence_period",
    )
    occurrence_timing = models.ForeignKey(
        "base.Timing",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="supply_delivery_occurrence",
    )
    supplier = models.ForeignKey(
        SupplyDeliverySupplierReference,
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="supply_delivery_supplier",
    )
    destination = models.ForeignKey(
        SupplyDeliveryDestinationReference,
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="supply_delivery_destination",
    )
    receiver = models.ManyToManyField(
        SupplyDeliveryReceiverReference,
        blank=True,
        related_name="supply_delivery_receiver",
    )



"""immunization app views."""

from django.http import Http404
from drf_spectacular.utils import extend_schema
from rest_framework import status
from rest_framework.permissions import AllowAny
from rest_framework.response import Response
from rest_framework.views import APIView

from dfhir.immunizations.models import Immunization
from dfhir.immunizations.serializers import ImmunizationSerializer


class ImmunizationListView(APIView):
    """Immunization list view."""

    permission_classes = [AllowAny]

    @extend_schema(responses={200: ImmunizationSerializer(many=True)})
    def get(self, request):
        """Get a list of immunizations."""
        immunizations = Immunization.objects.all()
        serializer = ImmunizationSerializer(immunizations, many=True)
        return Response(serializer.data, status=status.HTTP_200_OK)

    @extend_schema(
        request=ImmunizationSerializer, responses={201: ImmunizationSerializer}
    )
    def post(self, request):
        """Create a new immunization."""
        serializer = ImmunizationSerializer(data=request.data)
        serializer.is_valid(raise_exception=True)
        serializer.save()
        return Response(serializer.data, status=status.HTTP_201_CREATED)


class ImmunizationDetailView(APIView):
    """Immunization detail view."""

    permission_classes = [AllowAny]

    def get_object(self, pk):
        """Get an immunization object."""
        try:
            return Immunization.objects.get(pk=pk)
        except Immunization.DoesNotExist as err:
            raise Http404 from err

    @extend_schema(responses={200: ImmunizationSerializer})
    def get(self, request, pk):
        """Get an immunization."""
        immunization = self.get_object(pk)
        serializer = ImmunizationSerializer(immunization)
        return Response(serializer.data, status=status.HTTP_200_OK)

    @extend_schema(
        request=ImmunizationSerializer, responses={200: ImmunizationSerializer}
    )
    def patch(self, request, pk):
        """Update an immunization."""
        immunization = self.get_object(pk)
        serializer = ImmunizationSerializer(
            immunization, data=request.data, partial=True
        )
        serializer.is_valid(raise_exception=True)
        serializer.save()
        return Response(serializer.data, status=status.HTTP_200_OK)

    @extend_schema(responses={204: None})
    def delete(self, request, pk):
        """Delete an immunization."""
        immunization = self.get_object(pk)
        immunization.delete()
        return Response(status=status.HTTP_204_NO_CONTENT)



"""Encounter models."""

from django.db import models

from dfhir.base.models import (
    BaseReference,
    CodeableConcept,
    Identifier,
    OrganizationReference,
    Period,
    TimeStampedModel,
    VirtualServiceDetails,
)
from dfhir.base.models import (
    Quantity as Duration,
)
from dfhir.healthcareservices.models import (
    HealthCareServiceCodeableReference,
)

from .choices import EncounterLocationStatusChoices, EncounterStatus


class EncounterCondition(TimeStampedModel):
    """EncounterCondition model."""

    display = models.CharField(max_length=255, null=True)
    description = models.TextField(null=True)


class SpecialCourtesy(TimeStampedModel):
    """SpecialCourtesy model."""

    display = models.CharField(max_length=255, null=True)
    description = models.TextField(null=True)


class DietPreference(TimeStampedModel):
    """DietPreference model."""

    display = models.CharField(max_length=255, null=True)
    description = models.TextField(null=True)


class SpecialArrangement(TimeStampedModel):
    """SpecialArrangement model."""

    display = models.CharField(max_length=255, null=True)
    description = models.TextField(null=True)


class EncounterReference(BaseReference):
    """Encounter reference model."""

    identifier = models.ForeignKey(
        Identifier,
        on_delete=models.CASCADE,
        related_name="encounter_reference_identifier",
        null=True,
    )
    encounter = models.ForeignKey(
        "Encounter",
        on_delete=models.CASCADE,
        related_name="encounter_reference_encounter",
        null=True,
    )


class EncounterReasonValueReference(BaseReference):
    """Reference to the condition, diagnostic report, observation, immunization recommendation, procedure."""

    identifier = models.ForeignKey(
        Identifier,
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="encounter_reason_value_reference_identifier",
    )
    condition = models.ForeignKey(
        "conditions.Condition",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="encounter_reason_value_reference_condition",
    )
    diagnostic_report = models.ForeignKey(
        "diagnosticreports.DiagnosticReport",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="encounter_reason_value_reference_diagnostic_report",
    )
    observation = models.ForeignKey(
        "observations.Observation",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="encounter_reason_value_reference_observation",
    )
    immunization_recommendation = models.ForeignKey(
        "immunizationrecommendations.ImmunizationRecommendation",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="encounter_reason_value_reference_immunization_recommendation",
    )
    procedure = models.ForeignKey(
        "procedures.Procedure",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="encounter_reason_value_reference_procedure",
    )


class EncounterReasonValueCodeableReference(TimeStampedModel):
    """EncounterReasonCodeableReference model."""

    reference = models.ForeignKey(
        EncounterReasonValueReference,
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="encounter_reason_codeable_reference_reference",
    )
    concept = models.ForeignKey(
        CodeableConcept,
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="encounter_reason_codeable_reference_concept",
    )


class EncounterReason(TimeStampedModel):
    """EncounterReason model."""

    use = models.ManyToManyField(
        CodeableConcept, related_name="encounter_reason_use", blank=True
    )
    value = models.ManyToManyField(
        EncounterReasonValueCodeableReference,
        related_name="encounter_reason_value",
        blank=True,
    )


class EncounterDiagnosis(TimeStampedModel):
    """EncounterDiagnosis model."""

    use = models.ManyToManyField(
        CodeableConcept, related_name="encounter_diagnosis_use", blank=True
    )
    condition = models.ManyToManyField(
        "conditions.ConditionCodeableReference",
        related_name="encounter_diagnosis_condition",
        blank=True,
    )


class EncounterBasedOnReference(BaseReference):
    """Reference to the care plan, device request, medication request, service request, request orchestration, nutrition order, visual prescription, and immunization recommendation."""

    # TODO: Implement the commentetd models
    identifier = models.ForeignKey(
        Identifier,
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="encounter_based_on_reference_identifier",
    )
    care_plan = models.ForeignKey(
        "careplans.CarePlan",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="encounter_based_on_reference_care_plan",
    )
    device_request = models.ForeignKey(
        "devicerequests.DeviceRequest",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="encounter_based_on_reference_device_request",
    )
    medication_request = models.ForeignKey(
        "medicationrequests.MedicationRequest",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="encounter_based_on_reference_medication_request",
    )
    service_request = models.ForeignKey(
        "servicerequests.ServiceRequest",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="encounter_based_on_reference_service_request",
    )
    # request_orchestration = models.ForeignKey(
    #     "requestorchestrations.RequestOrchestration",
    #     on_delete=models.DO_NOTHING,
    #     null=True,
    #     related_name="encounter_based_on_reference_request_orchestration",
    # )
    nutrition_order = models.ForeignKey(
        "nutritionorders.NutritionOrder",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="encounter_based_on_reference_nutrition_order",
    )
    # visual_prescription = models.ForeignKey(
    #     "visualprescriptions.VisualPrescription",
    #     on_delete=models.DO_NOTHING,
    #     null=True,
    #     related_name="encounter_based_on_reference_visual_prescription",
    # )


class EncounterAdmission(TimeStampedModel):
    """EncounterAdmission model."""

    pre_admission_identifier = models.ForeignKey(
        Identifier,
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="encounter_admission_pre_admission_identifier",
    )

    origin = models.ForeignKey(
        "locations.LocationOrganizationReference",
        related_name="admission_origin",
        on_delete=models.DO_NOTHING,
        null=True,
    )
    admit_source = models.ForeignKey(
        CodeableConcept,
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="encounter_admission_admit_source",
    )
    re_admission = models.ForeignKey(
        CodeableConcept,
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="encounter_admission_re_admission",
    )
    destination = models.ForeignKey(
        "locations.LocationOrganizationReference",
        related_name="admission_destination",
        on_delete=models.CASCADE,
        null=True,
    )
    discharge_disposition = models.ForeignKey(
        CodeableConcept,
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="encounter_admission_discharge_disposition",
    )


class EncounterLocation(TimeStampedModel):
    """EncounterLocation model."""

    location = models.ForeignKey(
        "locations.LocationReference",
        on_delete=models.DO_NOTHING,
        related_name="encounter_location_location",
        null=True,
    )
    status = models.CharField(
        max_length=255, choices=EncounterLocationStatusChoices.choices, null=True
    )
    form = models.ForeignKey(
        CodeableConcept,
        on_delete=models.DO_NOTHING,
        related_name="encounter_location_form",
        null=True,
    )
    period = models.ForeignKey(
        Period,
        on_delete=models.DO_NOTHING,
        related_name="encounter_location_period",
        null=True,
    )


class EncounterParticipantActorReference(BaseReference):
    """EncounterParticipantActor model."""

    identifier = models.ForeignKey(
        Identifier,
        on_delete=models.DO_NOTHING,
        related_name="encounter_participant_actor_identifier",
        null=True,
    )
    patient = models.ForeignKey(
        "patients.Patient",
        on_delete=models.DO_NOTHING,
        related_name="encounter_participant_actor_patient",
        null=True,
    )
    group = models.ForeignKey(
        "groups.Group",
        on_delete=models.DO_NOTHING,
        related_name="encounter_participant_actor_group",
        null=True,
    )
    related_person = models.ForeignKey(
        "relatedpersons.RelatedPerson",
        on_delete=models.DO_NOTHING,
        related_name="encounter_participant_actor_related_person",
        null=True,
    )
    practitioner = models.ForeignKey(
        "practitioners.Practitioner",
        on_delete=models.DO_NOTHING,
        related_name="encounter_participant_actor_practitioner",
        null=True,
    )
    practitioner_role = models.ForeignKey(
        "practitionerroles.PractitionerRole",
        on_delete=models.DO_NOTHING,
        related_name="encounter_participant_actor_practitioner_role",
        null=True,
    )
    device = models.ForeignKey(
        "devices.Device",
        on_delete=models.DO_NOTHING,
        related_name="encounter_participant_actor_device",
        null=True,
    )
    healthcareservice = models.ForeignKey(
        "healthcareservices.HealthcareService",
        on_delete=models.DO_NOTHING,
        related_name="encounter_participant_actor_healthcareservice",
        null=True,
    )


class EncounterParticipant(TimeStampedModel):
    """Participants involved in the encounter."""

    actor = models.ForeignKey(
        EncounterParticipantActorReference,
        on_delete=models.CASCADE,
        null=True,
        related_name="participant_actor",
    )
    type = models.ManyToManyField(
        CodeableConcept, related_name="encounter_participant_type", blank=True
    )
    period = models.ForeignKey(
        Period,
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="encounter_participant_period",
    )


class Encounter(TimeStampedModel):
    """Encounter model."""

    identifier = models.ManyToManyField(
        Identifier, related_name="encounter_identifier", blank=True
    )
    status = models.CharField(
        max_length=255, choices=EncounterStatus.choices, null=True
    )
    klass = models.ManyToManyField(
        CodeableConcept, related_name="encounter_class", blank=True
    )
    priority = models.ForeignKey(
        CodeableConcept,
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="encounter_priority",
    )
    type = models.ManyToManyField(
        CodeableConcept, related_name="encounter_type", blank=True
    )
    service_type = models.ManyToManyField(
        HealthCareServiceCodeableReference, related_name="encounter_service_type"
    )
    subject = models.ForeignKey(
        "patients.PatientGroupReference",
        related_name="encounter_patient",
        on_delete=models.DO_NOTHING,
        null=True,
    )
    subject_status = models.ForeignKey(
        CodeableConcept,
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="encounter_subject_status",
    )
    episode_of_care = models.ManyToManyField(
        "episodeofcares.EpisodeOfCare",
        related_name="encounter_episode_of_care",
        blank=True,
    )
    based_on = models.ManyToManyField(
        EncounterBasedOnReference, related_name="encounter_based_on", blank=True
    )
    care_team = models.ManyToManyField(
        "careteams.CareTeamReference", related_name="encounter_care_team", blank=True
    )
    part_of = models.ManyToManyField(
        EncounterReference, related_name="encounter_part_of", blank=True
    )
    service_provider = models.ForeignKey(
        OrganizationReference,
        on_delete=models.DO_NOTHING,
        related_name="encounter_service_provider",
        null=True,
    )
    participant = models.ManyToManyField(
        EncounterParticipant, related_name="encounter_participant", blank=True
    )
    appointment = models.ManyToManyField(
        "appointments.AppointmentReference",
        related_name="encounter_appointment",
        blank=True,
    )
    virtual_service = models.ManyToManyField(
        VirtualServiceDetails, related_name="encounter_virtual_service", blank=True
    )
    actual_period = models.ForeignKey(
        Period,
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="encounter_actual_period",
    )
    planned_start_date = models.DateTimeField(null=True)
    planned_end_date = models.DateTimeField(null=True)
    # Quantity has the same fields as Duration and can be used interchangeably in this context
    length = models.ForeignKey(
        Duration,
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="encounter_length",
    )
    reason = models.ManyToManyField(
        EncounterReason, related_name="encounter_reason", blank=True
    )
    diagnosis = models.ManyToManyField(
        EncounterDiagnosis, related_name="encounter_diagnosis", blank=True
    )
    account = models.ManyToManyField(
        "accounts.AccountReference", related_name="encounter_account", blank=True
    )
    diet_preference = models.ManyToManyField(
        CodeableConcept, related_name="encounter_diet_preference", blank=True
    )
    special_arrangement = models.ManyToManyField(
        CodeableConcept, related_name="encounter_special_arrangement", blank=True
    )
    special_courtesy = models.ManyToManyField(
        CodeableConcept, related_name="encounter_special_courtesy", blank=True
    )
    admission = models.ForeignKey(
        EncounterAdmission,
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="encounter_admission",
    )
    location = models.ManyToManyField(
        EncounterLocation, related_name="encounter_location", blank=True
    )


class EncounterEpisodeOfCareReference(BaseReference):
    """EncounterEpisodeOfCareReference model."""

    identifier = models.ForeignKey(
        Identifier,
        on_delete=models.DO_NOTHING,
        related_name="encounter_episode_of_care_reference_identifier",
        null=True,
    )
    episode_of_care = models.ForeignKey(
        "episodeofcares.EpisodeOfCare",
        on_delete=models.DO_NOTHING,
        related_name="encounter_episode_of_care_reference_episode_of_care",
        null=True,
    )
    encounter = models.ForeignKey(
        Encounter,
        on_delete=models.DO_NOTHING,
        related_name="encounter_episode_of_care_reference_encounter",
        null=True,
    )



"""Claim responses serializers."""

from drf_writable_nested.serializers import WritableNestedModelSerializer

from dfhir.base.serializers import (
    AddressSerializer,
    AttachmentSerializer,
    BaseReferenceModelSerializer,
    BaseWritableNestedModelSerializer,
    CodeableConceptSerializer,
    IdentifierSerializer,
    MoneySerializer,
    OrganizationReferenceSerializer,
    PeriodSerializer,
    QuantitySerializer,
    SimpleQuantitySerializer,
)
from dfhir.bodystructures.serializers import BodyStructureCodeableReferenceSerializer
from dfhir.claims.serializers import ClaimReferenceSerializer
from dfhir.coverages.serializers import CoverageReferenceSerializer
from dfhir.encounters.serializers import EncounterReferenceSerializer
from dfhir.locations.serializers import LocationReferenceSerializer
from dfhir.patients.serializers import PatientReferenceSerializer

from .models import (
    ClaimResponse,
    ClaimResponseAddItem,
    ClaimResponseAddItemBodySite,
    ClaimResponseAddItemDetail,
    ClaimResponseAddItemDetailSubDetail,
    ClaimResponseAddItemProviderReference,
    ClaimResponseAddItemRequestReference,
    ClaimResponseError,
    ClaimResponseEvent,
    ClaimResponseInsurance,
    ClaimResponseItem,
    ClaimResponseItemAdjudication,
    ClaimResponseItemDetail,
    ClaimResponseItemDetailSubDetail,
    ClaimResponseItemReviewOutcome,
    ClaimResponsePayment,
    ClaimResponseProcessNote,
    ClaimResponseReference,
    ClaimResponseRequestorReference,
    ClaimResponseTotal,
)


class ClaimResponseReferenceSerializer(BaseReferenceModelSerializer):
    """ClaimResponse reference serializer."""

    identifier = IdentifierSerializer(many=False, required=False)

    class Meta:
        """Meta class."""

        model = ClaimResponseReference
        exclude = ["created_at", "updated_at"]


class ClaimResponseAddItemProviderReferenceReferenceSerializer(
    BaseReferenceModelSerializer
):
    """ClaimResponse Provider Reference serializer."""

    identifier = IdentifierSerializer(many=False, required=False)

    class Meta:
        """Meta class."""

        model = ClaimResponseAddItemProviderReference
        exclude = ["created_at", "updated_at"]


class ClaimResponseAddItemRequestReferenceReferenceSerializer(
    BaseReferenceModelSerializer
):
    """ClaimResponse Request Reference serializer."""

    identifier = IdentifierSerializer(many=False, required=False)

    class Meta:
        """Meta class."""

        model = ClaimResponseAddItemRequestReference
        exclude = ["created_at", "updated_at"]


class ClaimResponseItemAdjudicationSerializer(WritableNestedModelSerializer):
    """ClaimResponse Item Adjudication serializer."""

    category = CodeableConceptSerializer(many=False, required=False)
    reason = CodeableConceptSerializer(many=False, required=False)
    amount = MoneySerializer(many=False, required=False)
    quantity = QuantitySerializer(many=False, required=False)

    class Meta:
        """Meta class."""

        model = ClaimResponseItemAdjudication
        exclude = ["created_at", "updated_at"]


class ClaimResponseItemReviewOutcomeSerializer(WritableNestedModelSerializer):
    """ClaimResponse Item Review Outcome serializer."""

    decision = CodeableConceptSerializer(many=False, required=False)
    reason = CodeableConceptSerializer(many=True, required=False)
    pre_auth_period = PeriodSerializer(many=False, required=False)

    class Meta:
        """Meta class."""

        model = ClaimResponseItemReviewOutcome
        exclude = ["created_at", "updated_at"]


class ClaimResponseAddItemDetailSubDetailSerializer(WritableNestedModelSerializer):
    """ClaimResponse Add Item Detail Sub Detail serializer."""

    trace_number = IdentifierSerializer(many=True, required=False)
    revenue = CodeableConceptSerializer(many=False, required=False)
    product_or_service = CodeableConceptSerializer(many=False, required=False)
    product_or_service_end = CodeableConceptSerializer(many=False, required=False)
    modifier = CodeableConceptSerializer(many=True, required=False)
    quantity = SimpleQuantitySerializer(many=False, required=False)
    unit_price = MoneySerializer(many=False, required=False)
    tax = MoneySerializer(many=False, required=False)
    net = MoneySerializer(many=False, required=False)
    review_outcome = ClaimResponseItemReviewOutcomeSerializer(
        many=False, required=False
    )
    adjudication = ClaimResponseItemAdjudicationSerializer(many=True, required=False)

    class Meta:
        """Meta class."""

        model = ClaimResponseAddItemDetailSubDetail
        exclude = ["created_at", "updated_at"]


class ClaimResponseAddItemDetailSerializer(WritableNestedModelSerializer):
    """ClaimResponse Add Item Detail serializer."""

    trace_number = IdentifierSerializer(many=True, required=False)
    revenue = CodeableConceptSerializer(many=False, required=False)
    product_or_service = CodeableConceptSerializer(many=False, required=False)
    product_or_service_end = CodeableConceptSerializer(many=False, required=False)
    modifier = CodeableConceptSerializer(many=True, required=False)
    quantity = SimpleQuantitySerializer(many=False, required=False)
    unit_price = MoneySerializer(many=False, required=False)
    tax = MoneySerializer(many=False, required=False)
    net = MoneySerializer(many=False, required=False)
    review_outcome = ClaimResponseItemReviewOutcomeSerializer(
        many=False, required=False
    )
    adjudication = ClaimResponseItemAdjudicationSerializer(many=True, required=False)
    sub_detail = ClaimResponseAddItemDetailSubDetailSerializer(
        many=True, required=False
    )

    class Meta:
        """Meta class."""

        model = ClaimResponseAddItemDetail
        exclude = ["created_at", "updated_at"]


class ClaimResponseAddItemBodySiteSerializer(WritableNestedModelSerializer):
    """ClaimResponse Body Site Reference serializer."""

    site = BodyStructureCodeableReferenceSerializer(many=True, required=False)
    sub_site = CodeableConceptSerializer(many=True, required=False)

    class Meta:
        """Meta class."""

        model = ClaimResponseAddItemBodySite
        exclude = ["created_at", "updated_at"]


class ClaimResponseAddItemSerializer(WritableNestedModelSerializer):
    """ClaimResponse Add Item serializer."""

    trace_number = IdentifierSerializer(many=True, required=False)
    provider = ClaimResponseAddItemProviderReferenceReferenceSerializer(
        many=False, required=False
    )
    revenue = CodeableConceptSerializer(many=False, required=False)
    product_or_service = CodeableConceptSerializer(many=False, required=False)
    product_or_service_end = CodeableConceptSerializer(many=False, required=False)
    request = ClaimResponseAddItemRequestReferenceReferenceSerializer(
        many=True, required=False
    )
    modifier = CodeableConceptSerializer(many=True, required=False)
    program_code = CodeableConceptSerializer(many=True, required=False)
    serviced_period = PeriodSerializer(many=False, required=False)
    location_codeable_concept = CodeableConceptSerializer(many=False, required=False)
    location_address = AddressSerializer(many=False, required=False)
    location_reference = LocationReferenceSerializer(many=False, required=False)
    quantity = SimpleQuantitySerializer(many=False, required=False)
    unit_price = MoneySerializer(many=False, required=False)
    tax = MoneySerializer(many=False, required=False)
    net = MoneySerializer(many=False, required=False)
    body_site = ClaimResponseAddItemBodySiteSerializer(many=False, required=False)
    review_outcome = ClaimResponseItemReviewOutcomeSerializer(
        many=False, required=False
    )
    adjudication = ClaimResponseItemAdjudicationSerializer(many=True, required=False)
    detail = ClaimResponseAddItemDetailSerializer(many=True, required=False)

    class Meta:
        """Meta class."""

        model = ClaimResponseAddItem
        exclude = ["created_at", "updated_at"]


class ClaimResponseItemDetailSubDetailSerializer(WritableNestedModelSerializer):
    """ClaimResponse Item Detail Sub Detail serializer."""

    trace_number = IdentifierSerializer(many=True, required=False)
    review_outcome = ClaimResponseItemReviewOutcomeSerializer(
        many=False, required=False
    )
    adjudication = ClaimResponseItemAdjudicationSerializer(many=True, required=False)

    class Meta:
        """Meta class."""

        model = ClaimResponseItemDetailSubDetail
        exclude = ["created_at", "updated_at"]


class ClaimResponseItemDetailSerializer(WritableNestedModelSerializer):
    """ClaimResponse Item Detail serializer."""

    trace_number = IdentifierSerializer(many=True, required=False)
    review_outcome = ClaimResponseItemReviewOutcomeSerializer(
        many=False, required=False
    )
    adjudication = ClaimResponseItemAdjudicationSerializer(many=True, required=False)
    sub_detail = ClaimResponseItemDetailSubDetailSerializer(many=True, required=False)

    class Meta:
        """Meta class."""

        model = ClaimResponseItemDetail
        exclude = ["created_at", "updated_at"]


class ClaimResponseItemSerializer(WritableNestedModelSerializer):
    """ClaimResponse Item serializer."""

    trace_number = IdentifierSerializer(many=True, required=False)
    review_outcome = ClaimResponseItemReviewOutcomeSerializer(
        many=False, required=False
    )
    adjudication = ClaimResponseItemAdjudicationSerializer(many=True, required=False)
    detail = ClaimResponseItemDetailSerializer(many=True, required=False)

    class Meta:
        """Meta class."""

        model = ClaimResponseItem
        exclude = ["created_at", "updated_at"]


class ClaimResponseRequestorReferenceSerializer(BaseReferenceModelSerializer):
    """ClaimResponse Requestor reference serializer."""  # codespell:ignore requestor

    identifier = IdentifierSerializer(many=False, required=False)

    class Meta:
        """Meta class."""

        model = ClaimResponseRequestorReference
        exclude = ["created_at", "updated_at"]


class ClaimResponseErrorSerializer(WritableNestedModelSerializer):
    """ClaimResponse Error serializer."""

    code = CodeableConceptSerializer(many=False, required=False)

    class Meta:
        """Meta class."""

        model = ClaimResponseError
        exclude = ["created_at", "updated_at"]


class ClaimResponseEventSerializer(WritableNestedModelSerializer):
    """ClaimResponse Event serializer."""

    type = CodeableConceptSerializer(many=False, required=False)
    when_period = PeriodSerializer(many=False, required=False)

    class Meta:
        """Meta class."""

        model = ClaimResponseEvent
        exclude = ["created_at", "updated_at"]


class ClaimResponsePaymentSerializer(WritableNestedModelSerializer):
    """ClaimResponse Payment serializer."""

    type = CodeableConceptSerializer(many=False, required=False)
    adjustment = MoneySerializer(many=False, required=False)
    adjustment_reason = CodeableConceptSerializer(many=False, required=False)
    amount = MoneySerializer(many=False, required=False)
    identifier = IdentifierSerializer(many=False, required=False)

    class Meta:
        """Meta class."""

        model = ClaimResponsePayment
        exclude = ["created_at", "updated_at"]


class ClaimResponseTotalSerializer(WritableNestedModelSerializer):
    """ClaimResponse Total serializer."""

    category = CodeableConceptSerializer(many=False, required=False)
    amount = MoneySerializer(many=False, required=False)

    class Meta:
        """Meta class."""

        model = ClaimResponseTotal
        exclude = ["created_at", "updated_at"]


class ClaimResponseInsuranceSerializer(WritableNestedModelSerializer):
    """ClaimResponse Insurance serializer."""

    coverage = CoverageReferenceSerializer(many=False, required=False)
    claimResponse = ClaimResponseReferenceSerializer(many=False, required=False)

    class Meta:
        """Meta class."""

        model = ClaimResponseInsurance
        exclude = ["created_at", "updated_at"]


class ClaimResponseProcessNoteSerializer(WritableNestedModelSerializer):
    """ClaimResponse Process Note serializer."""

    type = CodeableConceptSerializer(many=False, required=False)
    language = CodeableConceptSerializer(many=False, required=False)

    class Meta:
        """Meta class."""

        model = ClaimResponseProcessNote
        exclude = ["created_at", "updated_at"]


class ClaimResponseSerializer(BaseWritableNestedModelSerializer):
    """ClaimResponse serializer."""

    identifier = IdentifierSerializer(many=True, required=False)
    trace_number = IdentifierSerializer(many=True, required=False)
    type = CodeableConceptSerializer(many=False, required=False)
    sub_type = CodeableConceptSerializer(many=False, required=False)
    patient = PatientReferenceSerializer(many=False, required=False)
    insurer = OrganizationReferenceSerializer(many=False, required=False)
    requestor = ClaimResponseRequestorReferenceSerializer(  # codespell:ignore requestor
        many=False, required=False
    )
    request = ClaimReferenceSerializer(many=False, required=False)
    decision = CodeableConceptSerializer(many=False, required=False)
    pre_auth_period = PeriodSerializer(many=False, required=False)
    event = ClaimResponseEventSerializer(many=True, required=False)
    payee_type = CodeableConceptSerializer(many=False, required=False)
    encounter = EncounterReferenceSerializer(many=False, required=False)
    diagnosis_related_group = CodeableConceptSerializer(many=False, required=False)
    item = ClaimResponseItemSerializer(many=True, required=False)
    add_item = ClaimResponseAddItemSerializer(many=True, required=False)
    adjudication = ClaimResponseItemAdjudicationSerializer(many=True, required=False)
    total = ClaimResponseTotalSerializer(many=True, required=False)
    payment = ClaimResponsePaymentSerializer(many=False, required=False)
    funds_reserve = CodeableConceptSerializer(many=False, required=False)
    form_code = CodeableConceptSerializer(many=False, required=False)
    form = AttachmentSerializer(many=False, required=False)
    process_note = ClaimResponseProcessNoteSerializer(many=True, required=False)
    # communication_request = CommunicationRequestReferenceSerializer(many=True, required=False)
    insurance = ClaimResponseInsuranceSerializer(many=True, required=False)
    error = ClaimResponseErrorSerializer(many=True, required=False)

    class Meta:
        """Meta class."""

        model = ClaimResponse
        exclude = ["created_at", "updated_at"]



"""Devicerequests models."""

from django.contrib.postgres.fields import ArrayField
from django.db import models

from dfhir.base.models import (
    Annotation,
    BaseReference,
    CodeableConcept,
    Identifier,
    Period,
    Quantity,
    Range,
    Reference,
    TimeStampedModel,
    Timing,
)

# from dfhir.coverages.models import CoverageClaimReference
# from dfhir.devices.models import DeviceDeviceDefinitionCodeableReference
from dfhir.provenances.models import ProvenanceReference

from . import choices


class DeviceRequestParameter(TimeStampedModel):
    """Device request parameter model."""

    code = models.ForeignKey(
        CodeableConcept,
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="device_request_parameter_code",
    )
    value_codeable_concept = models.ForeignKey(
        CodeableConcept,
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="device_request_parameter_value_codeable_concept",
    )
    value_quantity = models.ForeignKey(
        Quantity,
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="device_request_parameter_value_quantity",
    )
    value_range = models.ForeignKey(
        Range,
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="device_request_parameter_value_range",
    )
    value_boolean = models.BooleanField(default=False)


class DeviceRequestSubjectReference(BaseReference):
    """Device request subject reference model."""

    identifier = models.ForeignKey(
        Identifier,
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="device_request_subject_reference_identifier",
    )
    patient = models.ForeignKey(
        "patients.Patient",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="device_request_subject_reference_patient",
    )
    group = models.ForeignKey(
        "groups.Group",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="device_request_subject_reference_group",
    )
    location = models.ForeignKey(
        "locations.Location",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="device_request_subject_reference_location",
    )
    device = models.ForeignKey(
        "devices.Device",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="device_request_subject_reference_device",
    )


class DeviceRequestRequesterReference(BaseReference):
    """Device request requester reference model."""

    identifier = models.ForeignKey(
        Identifier,
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="device_request_requester_reference_identifier",
    )
    practitioner = models.ForeignKey(
        "practitioners.Practitioner",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="device_request_requester_reference_practitioner",
    )
    practitioner_role = models.ForeignKey(
        "practitionerroles.PractitionerRole",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="device_request_requester_reference_practitioner_role",
    )
    organization = models.ForeignKey(
        "organizations.Organization",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="device_request_requester_reference_organization",
    )
    device = models.ForeignKey(
        "devices.Device",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="device_request_requester_reference_device",
    )


class DeviceRequestRequesterCodeableReference(TimeStampedModel):
    """Device request requester codeable reference model."""

    reference = models.ForeignKey(
        DeviceRequestRequesterReference,
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="device_request_requester_codeable_reference_reference",
    )
    concept = models.ForeignKey(
        CodeableConcept,
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="device_request_requester_codeable_reference_code",
    )


class DeviceRequestReasonReference(BaseReference):
    """Device request reason reference model."""

    identifier = models.ForeignKey(
        Identifier,
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="device_request_reason_reference_identifier",
    )
    condition = models.ForeignKey(
        "conditions.Condition",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="device_request_reason_reference_condition",
    )
    observation = models.ForeignKey(
        "observations.Observation",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="device_request_reason_reference_observation",
    )
    diagnostic_report = models.ForeignKey(
        "diagnosticreports.DiagnosticReport",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="device_request_reason_reference_diagnostic_report",
    )
    document_reference = models.ForeignKey(
        "documentreferences.DocumentReference",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="device_request_reason_reference_document_reference",
    )


class DeviceRequestReasonCodeableReference(TimeStampedModel):
    """Device request reason codeable reference model."""

    reference = models.ForeignKey(
        DeviceRequestReasonReference,
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="device_request_reason_codeable_reference_reference",
    )
    concept = models.ForeignKey(
        CodeableConcept,
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="device_request_reason_codeable_reference_concept",
    )


class DeviceRequestPerformerReference(BaseReference):
    """Device request performer reference model."""

    identifier = models.ForeignKey(
        Identifier,
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="device_request_performer_reference_identifier",
    )
    practitioner = models.ForeignKey(
        "practitioners.Practitioner",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="device_request_performer_reference_practitioner",
    )
    practitioner_role = models.ForeignKey(
        "practitionerroles.PractitionerRole",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="device_request_performer_reference_practitioner_role",
    )
    organization = models.ForeignKey(
        "organizations.Organization",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="device_request_performer_reference_organization",
    )
    care_team = models.ForeignKey(
        "careteams.CareTeam",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="device_request_performer_reference_care_team",
    )
    healthcare_service = models.ForeignKey(
        "healthcareservices.HealthCareService",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="device_request_performer_reference_healthcare_service",
    )
    patient = models.ForeignKey(
        "patients.Patient",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="device_request_performer_reference_patient",
    )
    device = models.ForeignKey(
        "devices.Device",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="device_request_performer_reference_device",
    )
    related_person = models.ForeignKey(
        "relatedpersons.RelatedPerson",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="device_request_performer_reference_related_person",
    )


class DeviceRequestPerformerCodeableReference(TimeStampedModel):
    """Device request performer codeable reference model."""

    reference = models.ForeignKey(
        DeviceRequestPerformerReference,
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="device_request_performer_codeable_reference_reference",
    )
    concept = models.ForeignKey(
        CodeableConcept,
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="device_request_performer_codeable_reference_concept",
    )


class DeviceRequest(TimeStampedModel):
    """Device request model."""

    identifier = models.ManyToManyField(
        Identifier,
        blank=True,
        related_name="device_request_identifier",
    )
    # instantiates_canonical = models.ForeignKey(
    #     ActivityDefinitionPlanDefinitionCanonical,
    #     on_delete=models.DO_NOTHING,
    #     null=True,
    #     related_name="device_request_instantiates_canonical",
    # )
    instantiates_uri = ArrayField(models.URLField(), null=True)
    based_on = models.ManyToManyField(
        Reference,
        related_name="device_request_based_on",
        blank=True,
    )
    replaces = models.ManyToManyField(
        "DeviceRequestReference",
        related_name="device_request_replaces",
        blank=True,
    )
    group_identifier = models.ForeignKey(
        Identifier,
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="device_request_group_identifier",
    )

    status = models.CharField(
        max_length=255,
        choices=choices.DeviceRequestStatus.choices,
        default=choices.DeviceRequestStatus.ACTIVE,
    )
    intent = models.CharField(
        max_length=255,
        choices=choices.DeviceRequestIntent.choices,
        default=choices.DeviceRequestIntent.ORDER,
    )
    priority = models.CharField(
        max_length=255,
        choices=choices.DeviceRequestPriority.choices,
        default=choices.DeviceRequestPriority.ROUTINE,
    )
    do_not_perform = models.BooleanField(default=False)
    code = models.ForeignKey(
        "devices.DeviceDeviceDefinitionCodeableReference",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="device_request_code",
    )
    quantity = models.IntegerField(null=True)
    parameter = models.ManyToManyField(
        DeviceRequestParameter, related_name="device_request_parameter", blank=True
    )
    subject = models.ForeignKey(
        DeviceRequestSubjectReference,
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="device_request_subject",
    )
    encounter = models.ForeignKey(
        "encounters.EncounterReference",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="device_request_encounter",
    )
    occurrence_datetime = models.DateTimeField(null=True)
    occurrence_period = models.ForeignKey(
        Period,
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="device_request_occurrence_period",
    )
    occurrence_timing = models.ForeignKey(
        Timing,
        on_delete=models.CASCADE,
        related_name="device_request_occurrence_timing",
        null=True,
    )
    authored_on = models.DateTimeField(null=True)
    requester = models.ForeignKey(
        DeviceRequestRequesterReference,
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="device_request_requester",
    )
    performer = models.ForeignKey(
        DeviceRequestPerformerCodeableReference,
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="device_request_performer",
    )
    reason = models.ManyToManyField(
        DeviceRequestReasonCodeableReference,
        related_name="device_request_reason",
        blank=True,
    )
    as_needed = models.BooleanField(default=False)
    as_needed_for = models.ForeignKey(
        CodeableConcept,
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="device_request_as_needed_for",
    )
    # insurance = models.ManyToManyField(
    #     CoverageClaimReference,
    #     related_name="device_request_insurance",
    #     blank=True,
    # )
    supporting_info = models.ManyToManyField(
        Reference,
        related_name="device_request_supporting_info",
        blank=True,
    )
    note = models.ManyToManyField(
        Annotation,
        related_name="device_request_note",
        blank=True,
    )
    relevant_history = models.ManyToManyField(
        ProvenanceReference,
        related_name="device_request_relevant_history",
        blank=True,
    )


class DeviceRequestReference(BaseReference):
    """Device request reference model."""

    identifier = models.ForeignKey(
        Identifier,
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="device_request_reference_identifier",
    )
    device_request = models.ForeignKey(
        DeviceRequest,
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="device_request_reference_device_request",
    )



"""task model choices."""

from django.db import models
from django.utils.translation import gettext_lazy as _


class TaskStatusChoices(models.TextChoices):
    """task status choices."""

    DRAFT = "draft", _("Draft")
    REQUESTED = "requested", _("Requested")
    RECEIVED = "received", _("Received")
    ACCEPTED = "accepted", _("Accepted")
    REJECTED = "rejected", _("Rejected")
    READY = "ready", _("Ready")
    CANCELLED = "cancelled", _("Cancelled")
    IN_PROGRESS = "in-progress", _("In Progress")
    OM_HOLD = "on-hold", ("On Hold")
    FAILED = "failed", _("Failed")
    COMPLETED = "completed", _("Completed")
    ENTERED_IN_ERROR = "entered-in-error", _("Entered in Error")


class TaskIntentChoices(models.TextChoices):
    """task intent model choices."""

    UNKNOWN = "unknown", _("unknown")
    PROPOSAL = "proposal", _("proposal")
    PLAN = "plan", _("plan")
    ORDER = "order", _("order")
    ORIGINAL_ORDER = "original-order", _("original-order")
    REFLEX_ORDER = "reflex-order", _("reflex-order")
    FILLER_ORDER = "filler-order", _("filler-order")
    INSTANCE_ORDER = "instance-order", _("instance-order")
    OPTION = "option", _("option")


class TaskPriorityChoices(models.TextChoices):
    """task priority choices."""

    ROUTINE = "routine", _("routine")
    URGENT = "urgent", _("urgent")
    ASAP = "asap", _("asap")
    STAT = "stat", _("stat")



"""provenances views."""

from django.http import Http404
from drf_spectacular.utils import extend_schema
from rest_framework import status
from rest_framework.permissions import AllowAny
from rest_framework.response import Response
from rest_framework.views import APIView

from dfhir.provenances.models import Provenance
from dfhir.provenances.serializers import ProvenanceSerializer


class ProvenanceListView(APIView):
    """Provenance list view."""

    permission_classes = [AllowAny]
    serializer_class = ProvenanceSerializer

    @extend_schema(responses={200: ProvenanceSerializer(many=True)})
    def get(self, request):
        """Get all provenances."""
        provenances = Provenance.objects.all()
        serializer = self.serializer_class(provenances, many=True)
        return Response(serializer.data, status=status.HTTP_200_OK)

    @extend_schema(request=ProvenanceSerializer, responses={201: ProvenanceSerializer})
    def post(self, request):
        """Create a new provenance."""
        serializer = self.serializer_class(data=request.data)
        serializer.is_valid(raise_exception=True)
        serializer.save()
        return Response(serializer.data, status=status.HTTP_201_CREATED)


class ProvenanceDetailView(APIView):
    """Provenance detail view."""

    permission_classes = [AllowAny]
    serializer_class = ProvenanceSerializer

    def get_object(self, pk):
        """Get a provenance object."""
        try:
            return Provenance.objects.get(pk=pk)
        except Provenance.DoesNotExist as err:
            raise Http404 from err

    @extend_schema(responses={200: ProvenanceSerializer})
    def get(self, request, pk):
        """Get a provenance by ID."""
        provenance = self.get_object(pk)
        serializer = self.serializer_class(provenance)
        return Response(serializer.data, status=status.HTTP_200_OK)

    @extend_schema(request=ProvenanceSerializer, responses={200: ProvenanceSerializer})
    def patch(self, request, pk):
        """Update a provenance by ID."""
        provenance = self.get_object(pk)
        serializer = self.serializer_class(provenance, data=request.data, partial=True)
        serializer.is_valid(raise_exception=True)
        serializer.save()
        return Response(serializer.data, status=status.HTTP_200_OK)

    @extend_schema(responses={204: None})
    def delete(self, request, pk):
        """Delete a provenance by ID."""
        provenance = self.get_object(pk)
        provenance.delete()
        return Response(status=status.HTTP_204_NO_CONTENT)



# Generated by Django 5.2 on 2025-04-23 12:46

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('base', '0004_initial'),
        ('biologicallyderivedproducts', '0001_initial'),
        ('careteams', '0003_initial'),
        ('devicedefinitions', '0001_initial'),
        ('devices', '0001_initial'),
        ('endpoints', '0001_initial'),
        ('groups', '0001_initial'),
        ('healthcareservices', '0001_initial'),
        ('locations', '0001_initial'),
        ('medications', '0001_initial'),
        ('organizations', '0001_initial'),
        ('patients', '0001_initial'),
    ]

    operations = [
        migrations.CreateModel(
            name='PlanDefinitionActionInput',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('title', models.CharField(max_length=255, null=True)),
                ('related_data', models.CharField(max_length=255, null=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='PlanDefinitionActionOutput',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('title', models.CharField(max_length=255, null=True)),
                ('related_data', models.CharField(max_length=255, null=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='PlanDefinitionActor',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('title', models.CharField(max_length=255, null=True)),
                ('description', models.CharField(max_length=255, null=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='PlanDefinitionGoal',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='PlanDefinitionGoalTarget',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('detail_string', models.CharField(max_length=255, null=True)),
                ('detail_boolean', models.BooleanField(null=True)),
                ('detail_integer', models.IntegerField(null=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='PlanDefinitionSubjectReference',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('reference', models.CharField(max_length=255, null=True)),
                ('type', models.CharField(max_length=255, null=True)),
                ('display', models.CharField(max_length=255, null=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='PlanDefintionActorOption',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('type', models.CharField(choices=[('care-team', 'Care Team'), ('device', 'Device'), ('group', 'Group'), ('healthcareservice', 'Healthcare Service'), ('location', 'Location'), ('organization', 'Organization'), ('patient', 'Patient'), ('practitioner', 'Practitioner'), ('practitionerrole', 'Practitioner Role'), ('relatedperson', 'Related Person')], default='patient', max_length=255)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='PlanDefinitionActionCondition',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('kind', models.CharField(choices=[('applicability', 'Applicability'), ('start', 'Start'), ('stop', 'Stop')], default='applicability', max_length=255)),
                ('expression', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='plan_definition_action_condition_expression', to='base.expression')),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='PlanDefinitionActionDynamicValue',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('path', models.CharField(max_length=255, null=True)),
                ('expression', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='plan_definition_action_dynamic_value_expression', to='base.expression')),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='PlanDefinitionActionParticipant',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('type', models.CharField(choices=[('care-team', 'Care Team'), ('device', 'Device'), ('group', 'Group'), ('healthcareservice', 'Healthcare Service'), ('location', 'Location'), ('organization', 'Organization'), ('patient', 'Patient'), ('practitioner', 'Practitioner'), ('practitionerrole', 'Practitioner Role'), ('relatedperson', 'Related Person')], default='patient', max_length=255)),
                ('function', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='plan_definition_action_participant_function', to='base.codeableconcept')),
                ('role', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='plan_definition_action_participant_role', to='base.codeableconcept')),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='PlanDefinitionActionRelatedAction',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('target_id', models.CharField(max_length=255, null=True)),
                ('relationship', models.CharField(choices=[('before-start', 'Before Start'), ('before', 'Before'), ('before-end', 'Before End'), ('concurrent-with-start', 'Concurrent with Start'), ('concurrent', 'Concurrent'), ('concurrent-with-end', 'Concurrent with End'), ('after-start', 'After Start'), ('after', 'After'), ('after-end', 'After End')], default='before-start', max_length=255)),
                ('end_relationship', models.CharField(choices=[('before-start', 'Before Start'), ('before', 'Before'), ('before-end', 'Before End'), ('concurrent-with-start', 'Concurrent with Start'), ('concurrent', 'Concurrent'), ('concurrent-with-end', 'Concurrent with End'), ('after-start', 'After Start'), ('after', 'After'), ('after-end', 'After End')], default='before-start', max_length=255)),
                ('offset_duration', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='plan_definition_action_related_action_offset_duration', to='base.duration')),
                ('offset_range', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='plan_definition_action_related_action_offset_range', to='base.range')),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='PlanDefinitionAction',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('link_id', models.CharField(max_length=255, null=True)),
                ('prefix', models.CharField(max_length=255, null=True)),
                ('title', models.CharField(max_length=255, null=True)),
                ('description', models.CharField(max_length=255, null=True)),
                ('text_equivalent', models.CharField(max_length=255, null=True)),
                ('priority', models.CharField(choices=[('routine', 'Routine'), ('urgent', 'Urgent'), ('asap', 'As soon as possible'), ('stat', 'Stat')], default='routine', max_length=255)),
                ('goal_id', models.CharField(max_length=255, null=True)),
                ('grouping_behavior', models.CharField(choices=[('visual-group', 'Visual Group'), ('logical-group', 'Logical Group'), ('sentence-group', 'Sentence Group')], default='logical-group', max_length=255)),
                ('selection_behavior', models.CharField(choices=[('any', 'Any'), ('all', 'All'), ('all-or-none', 'All or None'), ('exactly-one', 'Exactly One'), ('at-most-one', 'At Most One'), ('one-or-more', 'One or More')], default='all', max_length=255)),
                ('required_behavior', models.CharField(choices=[('must', 'Must'), ('could', 'Could'), ('must-unless-documented', 'Must unless Documented')], default='must', max_length=255)),
                ('precheck_behavior', models.CharField(choices=[('yes', 'Yes'), ('no', 'No')], default='yes', max_length=255)),
                ('cardinality_behavior', models.CharField(choices=[('single', 'Single'), ('multiple', 'Multiple')], default='single', max_length=255)),
                ('definition_uri', models.CharField(max_length=255, null=True)),
                ('action', models.ManyToManyField(blank=True, to='plandefinitions.plandefinitionaction')),
                ('code', models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='plan_definition_action_code', to='base.codeableconcept')),
                ('documentation', models.ManyToManyField(blank=True, related_name='plan_definition_action_documentation', to='base.relatedartifact')),
                ('location', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='plan_definition_action_location', to='locations.locationcodeablereference')),
                ('reason', models.ManyToManyField(blank=True, related_name='plan_definition_action_reason', to='base.codeableconcept')),
                ('subject_codeable_concept', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='plan_definition_action_subject_codeable_concept', to='base.codeableconcept')),
                ('subject_reference', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='plan_definition_action_subject_reference', to='groups.groupreference')),
                ('timing_age', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='plan_definition_action_timing_age', to='base.age')),
                ('timing_duration', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='plan_definition_action_timing_duration', to='base.duration')),
                ('timing_range', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='plan_definition_action_timing_range', to='base.range')),
                ('timing_timing', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='plan_definition_action_timing_timing', to='base.timing')),
                ('trigger', models.ManyToManyField(blank=True, related_name='plan_definition_action_trigger', to='base.triggerdefinition')),
                ('type', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='plan_definition_action_type', to='base.codeableconcept')),
                ('condition', models.ManyToManyField(related_name='plan_definition_action_condition', to='plandefinitions.plandefinitionactioncondition')),
                ('dynamic_value', models.ManyToManyField(blank=True, related_name='plan_definition_action_dynamic_value', to='plandefinitions.plandefinitionactiondynamicvalue')),
                ('input', models.ManyToManyField(blank=True, related_name='plan_definition_action_input', to='plandefinitions.plandefinitionactioninput')),
                ('output', models.ManyToManyField(blank=True, related_name='plan_definition_action_output', to='plandefinitions.plandefinitionactionoutput')),
                ('participant', models.ManyToManyField(blank=True, related_name='plan_definition_action_participant', to='plandefinitions.plandefinitionactionparticipant')),
                ('related_action', models.ManyToManyField(blank=True, related_name='plan_definition_action_related_action', to='plandefinitions.plandefinitionactionrelatedaction')),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='PlanDefinition',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('url', models.URLField(null=True)),
                ('version', models.CharField(max_length=255, null=True)),
                ('version_algorithm_string', models.CharField(max_length=255, null=True)),
                ('name', models.CharField(max_length=255, null=True)),
                ('title', models.CharField(max_length=255, null=True)),
                ('subtitle', models.CharField(max_length=255, null=True)),
                ('status', models.CharField(choices=[('active', 'Active'), ('draft', 'Draft'), ('retired', 'Retired'), ('unknown', 'Unknown')], default='active', max_length=255)),
                ('experimental', models.BooleanField(default=False)),
                ('date', models.DateTimeField(null=True)),
                ('publisher', models.CharField(max_length=255, null=True)),
                ('description', models.TextField(null=True)),
                ('purpose', models.TextField(null=True)),
                ('usage', models.TextField(null=True)),
                ('copyright', models.CharField(max_length=255, null=True)),
                ('copyright_label', models.CharField(max_length=255, null=True)),
                ('approval_date', models.DateField(null=True)),
                ('last_review_date', models.DateField(null=True)),
                ('as_needed_boolean', models.BooleanField(null=True)),
                ('as_needed_codeable_concept', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='plan_definition_as_needed_codeable_concept', to='base.codeableconcept')),
                ('author', models.ManyToManyField(blank=True, related_name='plan_definition_author', to='base.contactdetail')),
                ('contact', models.ManyToManyField(blank=True, related_name='plan_definition_contact', to='base.contactdetail')),
                ('editor', models.ManyToManyField(blank=True, related_name='plan_definition_editor', to='base.contactdetail')),
                ('effective_period', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='plan_definition_effective_period', to='base.period')),
                ('endorser', models.ManyToManyField(blank=True, related_name='plan_definition_endorser', to='base.contactdetail')),
                ('identifier', models.ManyToManyField(blank=True, related_name='plan_definition_identifier', to='base.identifier')),
                ('jurisdiction', models.ManyToManyField(blank=True, related_name='plan_definition_jurisdiction', to='base.codeableconcept')),
                ('related_artifact', models.ManyToManyField(blank=True, related_name='plan_definition_related_artifact', to='base.relatedartifact')),
                ('reviewer', models.ManyToManyField(blank=True, related_name='plan_definition_reviewer', to='base.contactdetail')),
                ('subject_codeable_concept', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='plan_definition_subject_codeable_concept', to='base.codeableconcept')),
                ('topic', models.ManyToManyField(blank=True, related_name='plan_definition_topic', to='base.codeableconcept')),
                ('type', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='plan_definition_type', to='base.codeableconcept')),
                ('use_context', models.ManyToManyField(blank=True, related_name='plan_definition_use_context', to='base.usagecontext')),
                ('version_algorithm_coding', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='plan_definition_version_algorithm_coding', to='base.coding')),
                ('action', models.ManyToManyField(blank=True, related_name='plan_definition_action', to='plandefinitions.plandefinitionaction')),
                ('actor', models.ManyToManyField(blank=True, related_name='plan_definition_actor', to='plandefinitions.plandefinitionactor')),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='PlanDefinitionActorOptionTypeReferenceReference',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('reference', models.CharField(max_length=255, null=True)),
                ('type', models.CharField(max_length=255, null=True)),
                ('display', models.CharField(max_length=255, null=True)),
                ('biologically_derived_product', models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='plan_definition_actor_option_type_reference_reference_biologically_derived_product', to='biologicallyderivedproducts.biologicallyderivedproduct')),
                ('care_team', models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='plan_definition_actor_option_type_reference_reference_care_team', to='careteams.careteam')),
                ('device', models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='plan_definition_actor_option_type_reference_reference_device', to='devices.device')),
                ('device_definition', models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='plan_definition_actor_option_type_reference_reference_device_definition', to='devicedefinitions.devicedefinition')),
                ('endpoint', models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='plan_definition_actor_option_type_reference_reference_endpoint', to='endpoints.endpoint')),
                ('group', models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='plan_definition_actor_option_type_reference_reference_group', to='groups.group')),
                ('healthcare_service', models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='plan_definition_actor_option_type_reference_reference_healthcare_service', to='healthcareservices.healthcareservice')),
                ('identifier', models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='plan_definition_actor_option_type_reference_reference_identifier', to='base.identifier')),
                ('location', models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='plan_definition_actor_option_type_reference_reference_location', to='locations.location')),
                ('medication', models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='plan_definition_actor_option_type_reference_reference_medication', to='medications.medication')),
                ('organization', models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='plan_definition_actor_option_type_reference_reference_organization', to='organizations.organization')),
                ('patient', models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='plan_definition_actor_option_type_reference_reference_patient', to='patients.patient')),
            ],
            options={
                'abstract': False,
            },
        ),
    ]



"""Goals choices."""

from django.db import models
from django.utils.translation import gettext_lazy as _


class GoalLifecycleStatusChoices(models.TextChoices):
    """Goal Life Cycle status choices."""

    PROPOSED = "proposed", _("Proposed")
    PLANNED = "planned", _("Planned")
    ACCEPTED = "accepted", _("Accepted")
    ACTIVE = "active", _("Active")
    ON_HOLD = "on-hold", _("On Hold")
    COMPLETED = "completed", _("Completed")
    CANCELLED = "cancelled", _("Cancelled")
    ENTERED_IN_ERROR = "entered-in-error", _("Entered in Error")
    REJECTED = "rejected", _("Rejected")


class GoalAcceptanceStatusChoices(models.TextChoices):
    """Goal Acceptance Status choices."""

    AGREE = "agree", _("Agree")
    DISAGREE = "disagree", _("Disagree")
    PENDING = "pending", _("Pending")
